@@ -10,17 +10,17 @@
 mod error;
 
 use crate::error::ChrootError;
-use clap::{crate_version, App, Arg};
+use clap::{crate_version, Arg, Command};
 use std::ffi::CString;
 use std::io::Error;
 use std::path::Path;
-use std::process::Command;
+use std::process;
 use uucore::error::{set_exit_code, UResult};
 use uucore::libc::{self, chroot, setgid, setgroups, setuid};
-use uucore::{entries, InvalidEncodingHandling};
+use uucore::{entries, format_usage, InvalidEncodingHandling};
 
 static ABOUT: &str = "Run COMMAND with root directory set to NEWROOT.";
-static SYNTAX: &str = "[OPTION]... NEWROOT [COMMAND [ARG]...]";
+static USAGE: &str = "{} [OPTION]... NEWROOT [COMMAND [ARG]...]";
 
 mod options {
     pub const NEWROOT: &str = "newroot";
@@ -31,7 +31,7 @@ mod options {
     pub const COMMAND: &str = "command";
 }
 
-#[uucore_procs::gen_uumain]
+#[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let args = args
         .collect_str(InvalidEncodingHandling::ConvertLossy)
@@ -77,7 +77,10 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     // NOTE: Tests can only trigger code beyond this point if they're invoked with root permissions
     set_context(newroot, &matches)?;
 
-    let pstatus = match Command::new(chroot_command).args(chroot_args).status() {
+    let pstatus = match process::Command::new(chroot_command)
+        .args(chroot_args)
+        .status()
+    {
         Ok(status) => status,
         Err(e) => return Err(ChrootError::CommandFailed(command[0].to_string(), e).into()),
     };
@@ -91,40 +94,41 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     Ok(())
 }
 
-pub fn uu_app() -> App<'static, 'static> {
-    App::new(uucore::util_name())
+pub fn uu_app<'a>() -> Command<'a> {
+    Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
-        .usage(SYNTAX)
+        .override_usage(format_usage(USAGE))
+        .infer_long_args(true)
         .arg(
-            Arg::with_name(options::NEWROOT)
-                .hidden(true)
+            Arg::new(options::NEWROOT)
+                .hide(true)
                 .required(true)
                 .index(1),
         )
         .arg(
-            Arg::with_name(options::USER)
-                .short("u")
+            Arg::new(options::USER)
+                .short('u')
                 .long(options::USER)
                 .help("User (ID or name) to switch before running the program")
                 .value_name("USER"),
         )
         .arg(
-            Arg::with_name(options::GROUP)
-                .short("g")
+            Arg::new(options::GROUP)
+                .short('g')
                 .long(options::GROUP)
                 .help("Group (ID or name) to switch to")
                 .value_name("GROUP"),
         )
         .arg(
-            Arg::with_name(options::GROUPS)
-                .short("G")
+            Arg::new(options::GROUPS)
+                .short('G')
                 .long(options::GROUPS)
                 .help("Comma-separated list of groups to switch to")
                 .value_name("GROUP1,GROUP2..."),
         )
         .arg(
-            Arg::with_name(options::USERSPEC)
+            Arg::new(options::USERSPEC)
                 .long(options::USERSPEC)
                 .help(
                     "Colon-separated user and group to switch to. \
@@ -134,9 +138,9 @@ pub fn uu_app() -> App<'static, 'static> {
                 .value_name("USER:GROUP"),
         )
         .arg(
-            Arg::with_name(options::COMMAND)
-                .hidden(true)
-                .multiple(true)
+            Arg::new(options::COMMAND)
+                .hide(true)
+                .multiple_occurrences(true)
                 .index(2),
         )
 }
@@ -200,12 +204,12 @@ fn set_main_group(group: &str) -> UResult<()> {
 }
 
 #[cfg(any(target_vendor = "apple", target_os = "freebsd"))]
-fn set_groups(groups: Vec<libc::gid_t>) -> libc::c_int {
+fn set_groups(groups: &[libc::gid_t]) -> libc::c_int {
     unsafe { setgroups(groups.len() as libc::c_int, groups.as_ptr()) }
 }
 
 #[cfg(target_os = "linux")]
-fn set_groups(groups: Vec<libc::gid_t>) -> libc::c_int {
+fn set_groups(groups: &[libc::gid_t]) -> libc::c_int {
     unsafe { setgroups(groups.len() as libc::size_t, groups.as_ptr()) }
 }
 
@@ -219,7 +223,7 @@ fn set_groups_from_str(groups: &str) -> UResult<()> {
             };
             groups_vec.push(gid);
         }
-        let err = set_groups(groups_vec);
+        let err = set_groups(&groups_vec);
         if err != 0 {
             return Err(ChrootError::SetGroupsFailed(Error::last_os_error()).into());
         }
