@@ -1,9 +1,10 @@
 #![crate_name = "uu_csplit"]
+// spell-checker:ignore rustdoc
+#![allow(rustdoc::private_intra_doc_links)]
 
 #[macro_use]
 extern crate uucore;
-use clap::{crate_version, App, Arg, ArgMatches};
-use regex::Regex;
+
 use std::cmp::Ordering;
 use std::io::{self, BufReader};
 use std::{
@@ -11,16 +12,22 @@ use std::{
     io::{BufRead, BufWriter, Write},
 };
 
+use clap::{crate_version, Arg, ArgMatches, Command};
+use regex::Regex;
+use uucore::display::Quotable;
+use uucore::error::{FromIo, UResult};
+use uucore::{format_usage, InvalidEncodingHandling};
+
 mod csplit_error;
 mod patterns;
 mod split_name;
 
 use crate::csplit_error::CsplitError;
 use crate::split_name::SplitName;
-use uucore::InvalidEncodingHandling;
 
 static SUMMARY: &str = "split a file into sections determined by context lines";
 static LONG_HELP: &str = "Output pieces of FILE separated by PATTERN(s) to files 'xx00', 'xx01', ..., and output byte counts of each piece to standard output.";
+const USAGE: &str = "{} [OPTION]... FILE PATTERN...";
 
 mod options {
     pub const SUFFIX_FORMAT: &str = "suffix-format";
@@ -34,10 +41,6 @@ mod options {
     pub const PATTERN: &str = "pattern";
 }
 
-fn get_usage() -> String {
-    format!("{0} [OPTION]... FILE PATTERN...", executable!())
-}
-
 /// Command line options for csplit.
 pub struct CsplitOptions {
     split_name: crate::SplitName,
@@ -48,13 +51,13 @@ pub struct CsplitOptions {
 }
 
 impl CsplitOptions {
-    fn new(matches: &ArgMatches) -> CsplitOptions {
+    fn new(matches: &ArgMatches) -> Self {
         let keep_files = matches.is_present(options::KEEP_FILES);
         let quiet = matches.is_present(options::QUIET);
         let elide_empty_files = matches.is_present(options::ELIDE_EMPTY_FILES);
         let suppress_matched = matches.is_present(options::SUPPRESS_MATCHED);
 
-        CsplitOptions {
+        Self {
             split_name: crash_if_err!(
                 1,
                 SplitName::new(
@@ -76,12 +79,12 @@ impl CsplitOptions {
 /// # Errors
 ///
 /// - [`io::Error`] if there is some problem reading/writing from/to a file.
-/// - [`::CsplitError::LineOutOfRange`] if the line number pattern is larger than the number of input
+/// - [`CsplitError::LineOutOfRange`] if the line number pattern is larger than the number of input
 ///   lines.
-/// - [`::CsplitError::LineOutOfRangeOnRepetition`], like previous but after applying the pattern
+/// - [`CsplitError::LineOutOfRangeOnRepetition`], like previous but after applying the pattern
 ///   more than once.
-/// - [`::CsplitError::MatchNotFound`] if no line matched a regular expression.
-/// - [`::CsplitError::MatchNotFoundOnRepetition`], like previous but after applying the pattern
+/// - [`CsplitError::MatchNotFound`] if no line matched a regular expression.
+/// - [`CsplitError::MatchNotFoundOnRepetition`], like previous but after applying the pattern
 ///   more than once.
 pub fn csplit<T>(
     options: &CsplitOptions,
@@ -99,9 +102,9 @@ where
     input_iter.rewind_buffer();
     if let Some((_, line)) = input_iter.next() {
         split_writer.new_writer()?;
-        split_writer.writeln(line?)?;
+        split_writer.writeln(&line?)?;
         for (_, line) in input_iter {
-            split_writer.writeln(line?)?;
+            split_writer.writeln(&line?)?;
         }
         split_writer.finish_split();
     }
@@ -236,12 +239,12 @@ impl<'a> SplitWriter<'a> {
     }
 
     /// Writes the line to the current split, appending a newline character.
-    /// If [`dev_null`] is true, then the line is discarded.
+    /// If [`self.dev_null`] is true, then the line is discarded.
     ///
     /// # Errors
     ///
     /// Some [`io::Error`] may occur when attempting to write the line.
-    fn writeln(&mut self, line: String) -> io::Result<()> {
+    fn writeln(&mut self, line: &str) -> io::Result<()> {
         if !self.dev_null {
             match self.current_writer {
                 Some(ref mut current_writer) => {
@@ -257,8 +260,8 @@ impl<'a> SplitWriter<'a> {
     }
 
     /// Perform some operations after completing a split, i.e., either remove it
-    /// if the [`::ELIDE_EMPTY_FILES_OPT`] option is enabled, or print how much bytes were written
-    /// to it if [`::QUIET_OPT`] is disabled.
+    /// if the [`options::ELIDE_EMPTY_FILES`] option is enabled, or print how much bytes were written
+    /// to it if [`options::QUIET`] is disabled.
     ///
     /// # Errors
     ///
@@ -298,7 +301,7 @@ impl<'a> SplitWriter<'a> {
     ///
     /// In addition to errors reading/writing from/to a file, if the line number
     /// `n` is greater than the total available lines, then a
-    /// [`::CsplitError::LineOutOfRange`] error is returned.
+    /// [`CsplitError::LineOutOfRange`] error is returned.
     fn do_to_line<I>(
         &mut self,
         pattern_as_str: &str,
@@ -316,24 +319,25 @@ impl<'a> SplitWriter<'a> {
             let l = line?;
             match n.cmp(&(&ln + 1)) {
                 Ordering::Less => {
-                    if input_iter.add_line_to_buffer(ln, l).is_some() {
-                        panic!("the buffer is big enough to contain 1 line");
-                    }
+                    assert!(
+                        input_iter.add_line_to_buffer(ln, l).is_none(),
+                        "the buffer is big enough to contain 1 line"
+                    );
                     ret = Ok(());
                     break;
                 }
                 Ordering::Equal => {
-                    if !self.options.suppress_matched
-                        && input_iter.add_line_to_buffer(ln, l).is_some()
-                    {
-                        panic!("the buffer is big enough to contain 1 line");
-                    }
+                    assert!(
+                        self.options.suppress_matched
+                            || input_iter.add_line_to_buffer(ln, l).is_none(),
+                        "the buffer is big enough to contain 1 line"
+                    );
                     ret = Ok(());
                     break;
                 }
                 Ordering::Greater => (),
             }
-            self.writeln(l)?;
+            self.writeln(&l)?;
         }
         self.finish_split();
         ret
@@ -346,9 +350,9 @@ impl<'a> SplitWriter<'a> {
     /// # Errors
     ///
     /// In addition to errors reading/writing from/to a file, the following errors may be returned:
-    /// - if no line matched, an [`::CsplitError::MatchNotFound`].
+    /// - if no line matched, an [`CsplitError::MatchNotFound`].
     /// - if there are not enough lines to accommodate the offset, an
-    /// [`::CsplitError::LineOutOfRange`].
+    /// [`CsplitError::LineOutOfRange`].
     fn do_to_match<I>(
         &mut self,
         pattern_as_str: &str,
@@ -363,7 +367,7 @@ impl<'a> SplitWriter<'a> {
             // The offset is zero or positive, no need for a buffer on the lines read.
             // NOTE: drain the buffer of input_iter, no match should be done within.
             for line in input_iter.drain_buffer() {
-                self.writeln(line)?;
+                self.writeln(&line)?;
             }
             // retain the matching line
             input_iter.set_size_of_buffer(1);
@@ -374,12 +378,13 @@ impl<'a> SplitWriter<'a> {
                     match (self.options.suppress_matched, offset) {
                         // no offset, add the line to the next split
                         (false, 0) => {
-                            if input_iter.add_line_to_buffer(ln, l).is_some() {
-                                panic!("the buffer is big enough to contain 1 line");
-                            }
+                            assert!(
+                                input_iter.add_line_to_buffer(ln, l).is_none(),
+                                "the buffer is big enough to contain 1 line"
+                            );
                         }
                         // a positive offset, some more lines need to be added to the current split
-                        (false, _) => self.writeln(l)?,
+                        (false, _) => self.writeln(&l)?,
                         _ => (),
                     };
                     offset -= 1;
@@ -388,7 +393,7 @@ impl<'a> SplitWriter<'a> {
                     while offset > 0 {
                         match input_iter.next() {
                             Some((_, line)) => {
-                                self.writeln(line?)?;
+                                self.writeln(&line?)?;
                             }
                             None => {
                                 self.finish_split();
@@ -402,7 +407,7 @@ impl<'a> SplitWriter<'a> {
                     self.finish_split();
                     return Ok(());
                 }
-                self.writeln(l)?;
+                self.writeln(&l)?;
             }
         } else {
             // With a negative offset we use a buffer to keep the lines within the offset.
@@ -416,14 +421,15 @@ impl<'a> SplitWriter<'a> {
                 let l = line?;
                 if regex.is_match(&l) {
                     for line in input_iter.shrink_buffer_to_size() {
-                        self.writeln(line)?;
+                        self.writeln(&line)?;
                     }
                     if !self.options.suppress_matched {
                         // add 1 to the buffer size to make place for the matched line
                         input_iter.set_size_of_buffer(offset_usize + 1);
-                        if input_iter.add_line_to_buffer(ln, l).is_some() {
-                            panic!("should be big enough to hold every lines");
-                        }
+                        assert!(
+                            input_iter.add_line_to_buffer(ln, l).is_none(),
+                            "should be big enough to hold every lines"
+                        );
                     }
                     self.finish_split();
                     if input_iter.buffer_len() < offset_usize {
@@ -432,12 +438,12 @@ impl<'a> SplitWriter<'a> {
                     return Ok(());
                 }
                 if let Some(line) = input_iter.add_line_to_buffer(ln, l) {
-                    self.writeln(line)?;
+                    self.writeln(&line)?;
                 }
             }
             // no match, drain the buffer into the current split
             for line in input_iter.drain_buffer() {
-                self.writeln(line)?;
+                self.writeln(&line)?;
             }
         }
 
@@ -465,8 +471,8 @@ impl<I> InputSplitter<I>
 where
     I: Iterator<Item = (usize, io::Result<String>)>,
 {
-    fn new(iter: I) -> InputSplitter<I> {
-        InputSplitter {
+    fn new(iter: I) -> Self {
+        Self {
             iter,
             buffer: Vec::new(),
             rewind: false,
@@ -502,7 +508,7 @@ where
         self.size = size;
     }
 
-    /// Add a line to the buffer. If the buffer has [`size`] elements, then its head is removed and
+    /// Add a line to the buffer. If the buffer has [`self.size`] elements, then its head is removed and
     /// the new line is pushed to the buffer. The removed head is then available in the returned
     /// option.
     fn add_line_to_buffer(&mut self, ln: usize, line: String) -> Option<String> {
@@ -565,7 +571,7 @@ mod tests {
                 assert_eq!(input_splitter.add_line_to_buffer(0, line), None);
                 assert_eq!(input_splitter.buffer_len(), 1);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         match input_splitter.next() {
@@ -574,7 +580,7 @@ mod tests {
                 assert_eq!(input_splitter.add_line_to_buffer(1, line), None);
                 assert_eq!(input_splitter.buffer_len(), 2);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         match input_splitter.next() {
@@ -586,7 +592,7 @@ mod tests {
                 );
                 assert_eq!(input_splitter.buffer_len(), 2);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         input_splitter.rewind_buffer();
@@ -596,7 +602,7 @@ mod tests {
                 assert_eq!(line, String::from("bbb"));
                 assert_eq!(input_splitter.buffer_len(), 1);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         match input_splitter.next() {
@@ -604,7 +610,7 @@ mod tests {
                 assert_eq!(line, String::from("ccc"));
                 assert_eq!(input_splitter.buffer_len(), 0);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         match input_splitter.next() {
@@ -612,7 +618,7 @@ mod tests {
                 assert_eq!(line, String::from("ddd"));
                 assert_eq!(input_splitter.buffer_len(), 0);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         assert!(input_splitter.next().is_none());
@@ -637,7 +643,7 @@ mod tests {
                 assert_eq!(input_splitter.add_line_to_buffer(0, line), None);
                 assert_eq!(input_splitter.buffer_len(), 1);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         match input_splitter.next() {
@@ -646,7 +652,7 @@ mod tests {
                 assert_eq!(input_splitter.add_line_to_buffer(1, line), None);
                 assert_eq!(input_splitter.buffer_len(), 2);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         match input_splitter.next() {
@@ -655,7 +661,7 @@ mod tests {
                 assert_eq!(input_splitter.add_line_to_buffer(2, line), None);
                 assert_eq!(input_splitter.buffer_len(), 3);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         input_splitter.rewind_buffer();
@@ -666,7 +672,7 @@ mod tests {
                 assert_eq!(input_splitter.add_line_to_buffer(0, line), None);
                 assert_eq!(input_splitter.buffer_len(), 3);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         match input_splitter.next() {
@@ -674,7 +680,7 @@ mod tests {
                 assert_eq!(line, String::from("aaa"));
                 assert_eq!(input_splitter.buffer_len(), 2);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         match input_splitter.next() {
@@ -682,7 +688,7 @@ mod tests {
                 assert_eq!(line, String::from("bbb"));
                 assert_eq!(input_splitter.buffer_len(), 1);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         match input_splitter.next() {
@@ -690,7 +696,7 @@ mod tests {
                 assert_eq!(line, String::from("ccc"));
                 assert_eq!(input_splitter.buffer_len(), 0);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         match input_splitter.next() {
@@ -698,20 +704,20 @@ mod tests {
                 assert_eq!(line, String::from("ddd"));
                 assert_eq!(input_splitter.buffer_len(), 0);
             }
-            item @ _ => panic!("wrong item: {:?}", item),
+            item => panic!("wrong item: {:?}", item),
         };
 
         assert!(input_splitter.next().is_none());
     }
 }
 
-pub fn uumain(args: impl uucore::Args) -> i32 {
-    let usage = get_usage();
+#[uucore::main]
+pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let args = args
         .collect_str(InvalidEncodingHandling::Ignore)
         .accept_any();
 
-    let matches = uu_app().usage(&usage[..]).get_matches_from(args);
+    let matches = uu_app().get_matches_from(args);
 
     // get the file to split
     let file_name = matches.value_of(options::FILE).unwrap();
@@ -722,76 +728,80 @@ pub fn uumain(args: impl uucore::Args) -> i32 {
         .unwrap()
         .map(str::to_string)
         .collect();
-    let patterns = return_if_err!(1, patterns::get_patterns(&patterns[..]));
+    let patterns = patterns::get_patterns(&patterns[..])?;
     let options = CsplitOptions::new(&matches);
     if file_name == "-" {
         let stdin = io::stdin();
-        crash_if_err!(1, csplit(&options, patterns, stdin.lock()));
+        Ok(csplit(&options, patterns, stdin.lock())?)
     } else {
-        let file = return_if_err!(1, File::open(file_name));
-        let file_metadata = return_if_err!(1, file.metadata());
+        let file = File::open(file_name)
+            .map_err_context(|| format!("cannot access {}", file_name.quote()))?;
+        let file_metadata = file
+            .metadata()
+            .map_err_context(|| format!("cannot access {}", file_name.quote()))?;
         if !file_metadata.is_file() {
-            crash!(1, "'{}' is not a regular file", file_name);
+            return Err(CsplitError::NotRegularFile(file_name.to_string()).into());
         }
-        crash_if_err!(1, csplit(&options, patterns, BufReader::new(file)));
-    };
-    0
+        Ok(csplit(&options, patterns, BufReader::new(file))?)
+    }
 }
 
-pub fn uu_app() -> App<'static, 'static> {
-    App::new(executable!())
+pub fn uu_app<'a>() -> Command<'a> {
+    Command::new(uucore::util_name())
         .version(crate_version!())
         .about(SUMMARY)
+        .override_usage(format_usage(USAGE))
+        .infer_long_args(true)
         .arg(
-            Arg::with_name(options::SUFFIX_FORMAT)
-                .short("b")
+            Arg::new(options::SUFFIX_FORMAT)
+                .short('b')
                 .long(options::SUFFIX_FORMAT)
                 .value_name("FORMAT")
                 .help("use sprintf FORMAT instead of %02d"),
         )
         .arg(
-            Arg::with_name(options::PREFIX)
-                .short("f")
+            Arg::new(options::PREFIX)
+                .short('f')
                 .long(options::PREFIX)
                 .value_name("PREFIX")
                 .help("use PREFIX instead of 'xx'"),
         )
         .arg(
-            Arg::with_name(options::KEEP_FILES)
-                .short("k")
+            Arg::new(options::KEEP_FILES)
+                .short('k')
                 .long(options::KEEP_FILES)
                 .help("do not remove output files on errors"),
         )
         .arg(
-            Arg::with_name(options::SUPPRESS_MATCHED)
+            Arg::new(options::SUPPRESS_MATCHED)
                 .long(options::SUPPRESS_MATCHED)
                 .help("suppress the lines matching PATTERN"),
         )
         .arg(
-            Arg::with_name(options::DIGITS)
-                .short("n")
+            Arg::new(options::DIGITS)
+                .short('n')
                 .long(options::DIGITS)
                 .value_name("DIGITS")
                 .help("use specified number of digits instead of 2"),
         )
         .arg(
-            Arg::with_name(options::QUIET)
-                .short("s")
+            Arg::new(options::QUIET)
+                .short('s')
                 .long(options::QUIET)
                 .visible_alias("silent")
                 .help("do not print counts of output file sizes"),
         )
         .arg(
-            Arg::with_name(options::ELIDE_EMPTY_FILES)
-                .short("z")
+            Arg::new(options::ELIDE_EMPTY_FILES)
+                .short('z')
                 .long(options::ELIDE_EMPTY_FILES)
                 .help("remove empty output files"),
         )
-        .arg(Arg::with_name(options::FILE).hidden(true).required(true))
+        .arg(Arg::new(options::FILE).hide(true).required(true))
         .arg(
-            Arg::with_name(options::PATTERN)
-                .hidden(true)
-                .multiple(true)
+            Arg::new(options::PATTERN)
+                .hide(true)
+                .multiple_occurrences(true)
                 .required(true),
         )
         .after_help(LONG_HELP)
