@@ -11,7 +11,7 @@
 use chrono::{DateTime, FixedOffset, Local, Offset, Utc};
 #[cfg(windows)]
 use chrono::{Datelike, Timelike};
-use clap::{crate_version, App, Arg};
+use clap::{crate_version, Arg, Command};
 #[cfg(all(unix, not(target_os = "macos"), not(target_os = "redox")))]
 use libc::{clock_settime, timespec, CLOCK_REALTIME};
 use std::fs::File;
@@ -21,7 +21,7 @@ use uucore::display::Quotable;
 #[cfg(not(any(target_os = "macos", target_os = "redox")))]
 use uucore::error::FromIo;
 use uucore::error::{UResult, USimpleError};
-use uucore::show_error;
+use uucore::{format_usage, show_error};
 #[cfg(windows)]
 use winapi::{
     shared::minwindef::WORD,
@@ -38,8 +38,10 @@ const MINUTE: &str = "minute";
 const SECOND: &str = "second";
 const NS: &str = "ns";
 
-const NAME: &str = "date";
 const ABOUT: &str = "print or set the system date and time";
+const USAGE: &str = "\
+    {} [OPTION]... [+FORMAT]...
+    {} [OPTION]... [MMDDhhmm[[CC]YY][.ss]]";
 
 const OPT_DATE: &str = "date";
 const OPT_FORMAT: &str = "format";
@@ -111,11 +113,11 @@ enum Iso8601Format {
 impl<'a> From<&'a str> for Iso8601Format {
     fn from(s: &str) -> Self {
         match s {
-            HOURS | HOUR => Iso8601Format::Hours,
-            MINUTES | MINUTE => Iso8601Format::Minutes,
-            SECONDS | SECOND => Iso8601Format::Seconds,
-            NS => Iso8601Format::Ns,
-            DATE => Iso8601Format::Date,
+            HOURS | HOUR => Self::Hours,
+            MINUTES | MINUTE => Self::Minutes,
+            SECONDS | SECOND => Self::Seconds,
+            NS => Self::Ns,
+            DATE => Self::Date,
             // Should be caught by clap
             _ => panic!("Invalid format: {}", s),
         }
@@ -131,23 +133,18 @@ enum Rfc3339Format {
 impl<'a> From<&'a str> for Rfc3339Format {
     fn from(s: &str) -> Self {
         match s {
-            DATE => Rfc3339Format::Date,
-            SECONDS | SECOND => Rfc3339Format::Seconds,
-            NS => Rfc3339Format::Ns,
+            DATE => Self::Date,
+            SECONDS | SECOND => Self::Seconds,
+            NS => Self::Ns,
             // Should be caught by clap
             _ => panic!("Invalid format: {}", s),
         }
     }
 }
 
-#[uucore_procs::gen_uumain]
+#[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let syntax = format!(
-        "{0} [OPTION]... [+FORMAT]...
- {0} [OPTION]... [MMDDhhmm[[CC]YY][.ss]]",
-        NAME
-    );
-    let matches = uu_app().usage(&syntax[..]).get_matches_from(args);
+    let matches = uu_app().get_matches_from(args);
 
     let format = if let Some(form) = matches.value_of(OPT_FORMAT) {
         if !form.starts_with('+') {
@@ -257,70 +254,72 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     Ok(())
 }
 
-pub fn uu_app() -> App<'static, 'static> {
-    App::new(uucore::util_name())
+pub fn uu_app<'a>() -> Command<'a> {
+    Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
+        .override_usage(format_usage(USAGE))
+        .infer_long_args(true)
         .arg(
-            Arg::with_name(OPT_DATE)
-                .short("d")
+            Arg::new(OPT_DATE)
+                .short('d')
                 .long(OPT_DATE)
                 .takes_value(true)
                 .help("display time described by STRING, not 'now'"),
         )
         .arg(
-            Arg::with_name(OPT_FILE)
-                .short("f")
+            Arg::new(OPT_FILE)
+                .short('f')
                 .long(OPT_FILE)
                 .takes_value(true)
                 .help("like --date; once for each line of DATEFILE"),
         )
         .arg(
-            Arg::with_name(OPT_ISO_8601)
-                .short("I")
+            Arg::new(OPT_ISO_8601)
+                .short('I')
                 .long(OPT_ISO_8601)
                 .takes_value(true)
                 .help(ISO_8601_HELP_STRING),
         )
         .arg(
-            Arg::with_name(OPT_RFC_EMAIL)
-                .short("R")
+            Arg::new(OPT_RFC_EMAIL)
+                .short('R')
                 .long(OPT_RFC_EMAIL)
                 .help(RFC_5322_HELP_STRING),
         )
         .arg(
-            Arg::with_name(OPT_RFC_3339)
+            Arg::new(OPT_RFC_3339)
                 .long(OPT_RFC_3339)
                 .takes_value(true)
                 .help(RFC_3339_HELP_STRING),
         )
         .arg(
-            Arg::with_name(OPT_DEBUG)
+            Arg::new(OPT_DEBUG)
                 .long(OPT_DEBUG)
                 .help("annotate the parsed date, and warn about questionable usage to stderr"),
         )
         .arg(
-            Arg::with_name(OPT_REFERENCE)
-                .short("r")
+            Arg::new(OPT_REFERENCE)
+                .short('r')
                 .long(OPT_REFERENCE)
                 .takes_value(true)
                 .help("display the last modification time of FILE"),
         )
         .arg(
-            Arg::with_name(OPT_SET)
-                .short("s")
+            Arg::new(OPT_SET)
+                .short('s')
                 .long(OPT_SET)
                 .takes_value(true)
                 .help(OPT_SET_HELP_STRING),
         )
         .arg(
-            Arg::with_name(OPT_UNIVERSAL)
-                .short("u")
+            Arg::new(OPT_UNIVERSAL)
+                .short('u')
                 .long(OPT_UNIVERSAL)
                 .alias(OPT_UNIVERSAL_2)
                 .help("print or set Coordinated Universal Time (UTC)"),
         )
-        .arg(Arg::with_name(OPT_FORMAT).multiple(false))
+        .arg(Arg::new(OPT_FORMAT).multiple_occurrences(false))
 }
 
 /// Return the appropriate format string for the given settings.
@@ -377,9 +376,9 @@ fn set_system_datetime(_date: DateTime<Utc>) -> UResult<()> {
 #[cfg(all(unix, not(target_os = "macos"), not(target_os = "redox")))]
 /// System call to set date (unix).
 /// See here for more:
-/// https://doc.rust-lang.org/libc/i686-unknown-linux-gnu/libc/fn.clock_settime.html
-/// https://linux.die.net/man/3/clock_settime
-/// https://www.gnu.org/software/libc/manual/html_node/Time-Types.html
+/// `<https://doc.rust-lang.org/libc/i686-unknown-linux-gnu/libc/fn.clock_settime.html>`
+/// `<https://linux.die.net/man/3/clock_settime>`
+/// `<https://www.gnu.org/software/libc/manual/html_node/Time-Types.html>`
 fn set_system_datetime(date: DateTime<Utc>) -> UResult<()> {
     let timespec = timespec {
         tv_sec: date.timestamp() as _,
