@@ -1,6 +1,10 @@
 use std::char::from_digit;
+use std::ffi::OsStr;
 
-const SPECIAL_SHELL_CHARS: &str = "~`#$&*()|[]{};\\'\"<>?! ";
+// These are characters with special meaning in the shell (e.g. bash).
+// The first const contains characters that only have a special meaning when they appear at the beginning of a name.
+const SPECIAL_SHELL_CHARS_START: &[char] = &['~', '#'];
+const SPECIAL_SHELL_CHARS: &str = "`$&*()|[]{};\\'\"<>?! ";
 
 pub(super) enum QuotingStyle {
     Shell {
@@ -75,8 +79,8 @@ impl Iterator for EscapeOctal {
 }
 
 impl EscapeOctal {
-    fn from(c: char) -> EscapeOctal {
-        EscapeOctal {
+    fn from(c: char) -> Self {
+        Self {
             c,
             idx: 2,
             state: EscapeOctalState::Backslash,
@@ -198,6 +202,8 @@ fn shell_without_escape(name: &str, quotes: Quotes, show_control_chars: bool) ->
             }
         }
     }
+
+    must_quote = must_quote || name.starts_with(SPECIAL_SHELL_CHARS_START);
     (escaped_str, must_quote)
 }
 
@@ -246,22 +252,25 @@ fn shell_with_escape(name: &str, quotes: Quotes) -> (String, bool) {
             }
         }
     }
+    must_quote = must_quote || name.starts_with(SPECIAL_SHELL_CHARS_START);
     (escaped_str, must_quote)
 }
 
-pub(super) fn escape_name(name: &str, style: &QuotingStyle) -> String {
+pub(super) fn escape_name(name: &OsStr, style: &QuotingStyle) -> String {
     match style {
         QuotingStyle::Literal { show_control } => {
             if !show_control {
-                name.chars()
+                name.to_string_lossy()
+                    .chars()
                     .flat_map(|c| EscapedChar::new_literal(c).hide_control())
                     .collect()
             } else {
-                name.into()
+                name.to_string_lossy().into_owned()
             }
         }
         QuotingStyle::C { quotes } => {
             let escaped_str: String = name
+                .to_string_lossy()
                 .chars()
                 .flat_map(|c| EscapedChar::new_c(c, *quotes))
                 .collect();
@@ -277,7 +286,8 @@ pub(super) fn escape_name(name: &str, style: &QuotingStyle) -> String {
             always_quote,
             show_control,
         } => {
-            let (quotes, must_quote) = if name.contains('"') {
+            let name = name.to_string_lossy();
+            let (quotes, must_quote) = if name.contains(&['"', '`', '$', '\\'][..]) {
                 (Quotes::Single, true)
             } else if name.contains('\'') {
                 (Quotes::Double, true)
@@ -288,9 +298,9 @@ pub(super) fn escape_name(name: &str, style: &QuotingStyle) -> String {
             };
 
             let (escaped_str, contains_quote_chars) = if *escape {
-                shell_with_escape(name, quotes)
+                shell_with_escape(&name, quotes)
             } else {
-                shell_without_escape(name, quotes, *show_control)
+                shell_without_escape(&name, quotes, *show_control)
             };
 
             match (must_quote | contains_quote_chars, quotes) {
@@ -353,10 +363,10 @@ mod tests {
         }
     }
 
-    fn check_names(name: &str, map: Vec<(&str, &str)>) {
+    fn check_names(name: &str, map: &[(&str, &str)]) {
         assert_eq!(
             map.iter()
-                .map(|(_, style)| escape_name(name, &get_style(style)))
+                .map(|(_, style)| escape_name(name.as_ref(), &get_style(style)))
                 .collect::<Vec<String>>(),
             map.iter()
                 .map(|(correct, _)| correct.to_string())
@@ -368,7 +378,7 @@ mod tests {
     fn test_simple_names() {
         check_names(
             "one_two",
-            vec![
+            &[
                 ("one_two", "literal"),
                 ("one_two", "literal-show"),
                 ("one_two", "escape"),
@@ -387,7 +397,7 @@ mod tests {
     fn test_spaces() {
         check_names(
             "one two",
-            vec![
+            &[
                 ("one two", "literal"),
                 ("one two", "literal-show"),
                 ("one\\ two", "escape"),
@@ -403,7 +413,7 @@ mod tests {
 
         check_names(
             " one",
-            vec![
+            &[
                 (" one", "literal"),
                 (" one", "literal-show"),
                 ("\\ one", "escape"),
@@ -423,7 +433,7 @@ mod tests {
         // One double quote
         check_names(
             "one\"two",
-            vec![
+            &[
                 ("one\"two", "literal"),
                 ("one\"two", "literal-show"),
                 ("one\"two", "escape"),
@@ -440,7 +450,7 @@ mod tests {
         // One single quote
         check_names(
             "one\'two",
-            vec![
+            &[
                 ("one'two", "literal"),
                 ("one'two", "literal-show"),
                 ("one'two", "escape"),
@@ -457,7 +467,7 @@ mod tests {
         // One single quote and one double quote
         check_names(
             "one'two\"three",
-            vec![
+            &[
                 ("one'two\"three", "literal"),
                 ("one'two\"three", "literal-show"),
                 ("one'two\"three", "escape"),
@@ -474,7 +484,7 @@ mod tests {
         // Consecutive quotes
         check_names(
             "one''two\"\"three",
-            vec![
+            &[
                 ("one''two\"\"three", "literal"),
                 ("one''two\"\"three", "literal-show"),
                 ("one''two\"\"three", "escape"),
@@ -494,7 +504,7 @@ mod tests {
         // A simple newline
         check_names(
             "one\ntwo",
-            vec![
+            &[
                 ("one?two", "literal"),
                 ("one\ntwo", "literal-show"),
                 ("one\\ntwo", "escape"),
@@ -511,7 +521,7 @@ mod tests {
         // A control character followed by a special shell character
         check_names(
             "one\n&two",
-            vec![
+            &[
                 ("one?&two", "literal"),
                 ("one\n&two", "literal-show"),
                 ("one\\n&two", "escape"),
@@ -529,7 +539,7 @@ mod tests {
         // no importance for file names.
         check_names(
             "\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F",
-            vec![
+            &[
                 ("????????????????", "literal"),
                 (
                     "\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0A\x0B\x0C\x0D\x0E\x0F",
@@ -567,7 +577,7 @@ mod tests {
         // The last 16 control characters.
         check_names(
             "\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1A\x1B\x1C\x1D\x1E\x1F",
-            vec![
+            &[
                 ("????????????????", "literal"),
                 (
                     "\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1A\x1B\x1C\x1D\x1E\x1F",
@@ -605,7 +615,7 @@ mod tests {
         // DEL
         check_names(
             "\x7F",
-            vec![
+            &[
                 ("?", "literal"),
                 ("\x7F", "literal-show"),
                 ("\\177", "escape"),
@@ -627,7 +637,7 @@ mod tests {
         // in other tests)
         check_names(
             "one?two",
-            vec![
+            &[
                 ("one?two", "literal"),
                 ("one?two", "literal-show"),
                 ("one?two", "escape"),
@@ -647,7 +657,7 @@ mod tests {
         // Escaped in C-style, but not in Shell-style escaping
         check_names(
             "one\\two",
-            vec![
+            &[
                 ("one\\two", "literal"),
                 ("one\\two", "literal-show"),
                 ("one\\\\two", "escape"),
@@ -659,4 +669,62 @@ mod tests {
             ],
         );
     }
+
+    #[test]
+    fn test_tilde_and_hash() {
+        check_names("~", &[("'~'", "shell"), ("'~'", "shell-escape")]);
+        check_names(
+            "~name",
+            &[("'~name'", "shell"), ("'~name'", "shell-escape")],
+        );
+        check_names(
+            "some~name",
+            &[("some~name", "shell"), ("some~name", "shell-escape")],
+        );
+        check_names("name~", &[("name~", "shell"), ("name~", "shell-escape")]);
+
+        check_names("#", &[("'#'", "shell"), ("'#'", "shell-escape")]);
+        check_names(
+            "#name",
+            &[("'#name'", "shell"), ("'#name'", "shell-escape")],
+        );
+        check_names(
+            "some#name",
+            &[("some#name", "shell"), ("some#name", "shell-escape")],
+        );
+        check_names("name#", &[("name#", "shell"), ("name#", "shell-escape")]);
+    }
+
+    #[test]
+    fn test_special_chars_in_double_quotes() {
+        check_names(
+            "can'$t",
+            &[
+                ("'can'\\''$t'", "shell"),
+                ("'can'\\''$t'", "shell-always"),
+                ("'can'\\''$t'", "shell-escape"),
+                ("'can'\\''$t'", "shell-escape-always"),
+            ],
+        );
+
+        check_names(
+            "can'`t",
+            &[
+                ("'can'\\''`t'", "shell"),
+                ("'can'\\''`t'", "shell-always"),
+                ("'can'\\''`t'", "shell-escape"),
+                ("'can'\\''`t'", "shell-escape-always"),
+            ],
+        );
+
+        check_names(
+            "can'\\t",
+            &[
+                ("'can'\\''\\t'", "shell"),
+                ("'can'\\''\\t'", "shell-always"),
+                ("'can'\\''\\t'", "shell-escape"),
+                ("'can'\\''\\t'", "shell-escape-always"),
+            ],
+        );
+    }
 }
