@@ -10,7 +10,7 @@
 #[macro_use]
 extern crate uucore;
 
-use clap::{crate_version, App, AppSettings, Arg};
+use clap::{crate_version, Arg, Command};
 use libc::{c_char, dup2, execvp, signal};
 use libc::{SIGHUP, SIG_IGN};
 use std::env;
@@ -22,7 +22,7 @@ use std::os::unix::prelude::*;
 use std::path::{Path, PathBuf};
 use uucore::display::Quotable;
 use uucore::error::{set_exit_code, UError, UResult};
-use uucore::InvalidEncodingHandling;
+use uucore::{format_usage, InvalidEncodingHandling};
 
 static ABOUT: &str = "Run COMMAND ignoring hangup signals.";
 static LONG_HELP: &str = "
@@ -31,6 +31,9 @@ If standard output is terminal, it'll be appended to nohup.out instead,
 or $HOME/nohup.out, if nohup.out open failed.
 If standard error is terminal, it'll be redirected to stdout.
 ";
+const USAGE: &str = "\
+    {} COMMAND [ARG]...
+    {} FLAG";
 static NOHUP_OUT: &str = "nohup.out";
 // exit codes that match the GNU implementation
 static EXIT_CANCELED: i32 = 125;
@@ -55,7 +58,7 @@ impl std::error::Error for NohupError {}
 impl UError for NohupError {
     fn code(&self) -> i32 {
         match self {
-            NohupError::OpenFailed(code, _) | NohupError::OpenFailed2(code, _, _, _) => *code,
+            Self::OpenFailed(code, _) | Self::OpenFailed2(code, _, _, _) => *code,
             _ => 2,
         }
     }
@@ -64,9 +67,9 @@ impl UError for NohupError {
 impl Display for NohupError {
     fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
         match self {
-            NohupError::CannotDetach => write!(f, "Cannot detach from console"),
-            NohupError::CannotReplace(s, e) => write!(f, "Cannot replace {}: {}", s, e),
-            NohupError::OpenFailed(_, e) => {
+            Self::CannotDetach => write!(f, "Cannot detach from console"),
+            Self::CannotReplace(s, e) => write!(f, "Cannot replace {}: {}", s, e),
+            Self::OpenFailed(_, e) => {
                 write!(f, "failed to open {}: {}", NOHUP_OUT.quote(), e)
             }
             NohupError::OpenFailed2(_, e1, s, e2) => write!(
@@ -81,14 +84,13 @@ impl Display for NohupError {
     }
 }
 
-#[uucore_procs::gen_uumain]
+#[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let usage = usage();
     let args = args
         .collect_str(InvalidEncodingHandling::ConvertLossy)
         .accept_any();
 
-    let matches = uu_app().usage(&usage[..]).get_matches_from(args);
+    let matches = uu_app().get_matches_from(args);
 
     replace_fds()?;
 
@@ -114,18 +116,20 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     Ok(())
 }
 
-pub fn uu_app() -> App<'static, 'static> {
-    App::new(uucore::util_name())
+pub fn uu_app<'a>() -> Command<'a> {
+    Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
         .after_help(LONG_HELP)
+        .override_usage(format_usage(USAGE))
         .arg(
-            Arg::with_name(options::CMD)
-                .hidden(true)
+            Arg::new(options::CMD)
+                .hide(true)
                 .required(true)
-                .multiple(true),
+                .multiple_occurrences(true),
         )
-        .setting(AppSettings::TrailingVarArg)
+        .trailing_var_arg(true)
+        .infer_long_args(true)
 }
 
 fn replace_fds() -> UResult<()> {
@@ -204,13 +208,6 @@ fn find_stdout() -> UResult<File> {
     }
 }
 
-fn usage() -> String {
-    format!(
-        "{0} COMMAND [ARG]...\n    {0} FLAG",
-        uucore::execution_phrase()
-    )
-}
-
 #[cfg(target_vendor = "apple")]
 extern "C" {
     fn _vprocmgr_detach_from_console(flags: u32) -> *const libc::c_int;
