@@ -7,25 +7,29 @@
 
 // spell-checker:ignore (ToDOs) corasick memchr Roff trunc oset iset
 
-#[macro_use]
-extern crate uucore;
-
-use clap::{crate_version, App, Arg};
+use clap::{crate_version, Arg, Command};
 use regex::Regex;
 use std::cmp;
 use std::collections::{BTreeSet, HashMap, HashSet};
 use std::default::Default;
+use std::error::Error;
+use std::fmt::{Display, Formatter};
 use std::fs::File;
 use std::io::{stdin, stdout, BufRead, BufReader, BufWriter, Read, Write};
-use uucore::InvalidEncodingHandling;
+use std::num::ParseIntError;
+use uucore::display::Quotable;
+use uucore::error::{FromIo, UError, UResult};
+use uucore::{format_usage, InvalidEncodingHandling};
 
 static NAME: &str = "ptx";
-static BRIEF: &str = "Usage: ptx [OPTION]... [INPUT]...   (without -G) or: \
-                 ptx -G [OPTION]... [INPUT [OUTPUT]] \n Output a permuted index, \
-                 including context, of the words in the input files. \n\n Mandatory \
-                 arguments to long options are mandatory for short options too.\n
-                 With no FILE, or when FILE is -, read standard input. \
-                Default is '-F /'.";
+const USAGE: &str = "\
+    {} [OPTION]... [INPUT]...
+    {} -G [OPTION]... [INPUT [OUTPUT]]";
+
+const ABOUT: &str = "\
+    Output a permuted index, including context, of the words in the input files. \n\n\
+    Mandatory arguments to long options are mandatory for short options too.\n\
+    With no FILE, or when FILE is -, read standard input. Default is '-F /'.";
 
 #[derive(Debug)]
 enum OutFormat {
@@ -50,8 +54,8 @@ struct Config {
 }
 
 impl Default for Config {
-    fn default() -> Config {
-        Config {
+    fn default() -> Self {
+        Self {
             format: OutFormat::Dumb,
             gnu_ext: true,
             auto_ref: false,
@@ -67,17 +71,21 @@ impl Default for Config {
     }
 }
 
-fn read_word_filter_file(matches: &clap::ArgMatches, option: &str) -> HashSet<String> {
+fn read_word_filter_file(
+    matches: &clap::ArgMatches,
+    option: &str,
+) -> std::io::Result<HashSet<String>> {
     let filename = matches
         .value_of(option)
         .expect("parsing options failed!")
         .to_string();
-    let reader = BufReader::new(crash_if_err!(1, File::open(filename)));
+    let file = File::open(filename)?;
+    let reader = BufReader::new(file);
     let mut words: HashSet<String> = HashSet::new();
     for word in reader.lines() {
-        words.insert(crash_if_err!(1, word));
+        words.insert(word?);
     }
-    words
+    Ok(words)
 }
 
 #[derive(Debug)]
@@ -90,19 +98,23 @@ struct WordFilter {
 }
 
 impl WordFilter {
-    fn new(matches: &clap::ArgMatches, config: &Config) -> WordFilter {
+    fn new(matches: &clap::ArgMatches, config: &Config) -> UResult<Self> {
         let (o, oset): (bool, HashSet<String>) = if matches.is_present(options::ONLY_FILE) {
-            (true, read_word_filter_file(matches, options::ONLY_FILE))
+            let words =
+                read_word_filter_file(matches, options::ONLY_FILE).map_err_context(String::new)?;
+            (true, words)
         } else {
             (false, HashSet::new())
         };
         let (i, iset): (bool, HashSet<String>) = if matches.is_present(options::IGNORE_FILE) {
-            (true, read_word_filter_file(matches, options::IGNORE_FILE))
+            let words = read_word_filter_file(matches, options::IGNORE_FILE)
+                .map_err_context(String::new)?;
+            (true, words)
         } else {
             (false, HashSet::new())
         };
         if matches.is_present(options::BREAK_FILE) {
-            crash!(1, "-b not implemented yet");
+            return Err(PtxError::NotImplemented("-b").into());
         }
         // Ignore empty string regex from cmd-line-args
         let arg_reg: Option<String> = if matches.is_present(options::WORD_REGEXP) {
@@ -129,13 +141,13 @@ impl WordFilter {
                 }
             }
         };
-        WordFilter {
+        Ok(Self {
             only_specified: o,
             ignore_specified: i,
             only_set: oset,
             ignore_set: iset,
             word_regex: reg,
-        }
+        })
     }
 }
 
@@ -149,7 +161,29 @@ struct WordRef {
     filename: String,
 }
 
-fn get_config(matches: &clap::ArgMatches) -> Config {
+#[derive(Debug)]
+enum PtxError {
+    DumbFormat,
+    NotImplemented(&'static str),
+    ParseError(ParseIntError),
+}
+
+impl Error for PtxError {}
+impl UError for PtxError {}
+
+impl Display for PtxError {
+    fn fmt(&self, f: &mut Formatter) -> std::fmt::Result {
+        match self {
+            Self::DumbFormat => {
+                write!(f, "There is no dumb format with GNU extensions disabled")
+            }
+            Self::NotImplemented(s) => write!(f, "{} not implemented yet", s),
+            Self::ParseError(e) => e.fmt(f),
+        }
+    }
+}
+
+fn get_config(matches: &clap::ArgMatches) -> UResult<Config> {
     let mut config: Config = Default::default();
     let err_msg = "parsing options failed";
     if matches.is_present(options::TRADITIONAL) {
@@ -157,10 +191,10 @@ fn get_config(matches: &clap::ArgMatches) -> Config {
         config.format = OutFormat::Roff;
         config.context_regex = "[^ \t\n]+".to_owned();
     } else {
-        crash!(1, "GNU extensions not implemented yet");
+        return Err(PtxError::NotImplemented("GNU extensions").into());
     }
     if matches.is_present(options::SENTENCE_REGEXP) {
-        crash!(1, "-S not implemented yet");
+        return Err(PtxError::NotImplemented("-S").into());
     }
     config.auto_ref = matches.is_present(options::AUTO_REFERENCE);
     config.input_ref = matches.is_present(options::REFERENCES);
@@ -179,15 +213,18 @@ fn get_config(matches: &clap::ArgMatches) -> Config {
             .to_string();
     }
     if matches.is_present(options::WIDTH) {
-        let width_str = matches.value_of(options::WIDTH).expect(err_msg).to_string();
-        config.line_width = crash_if_err!(1, (&width_str).parse::<usize>());
+        config.line_width = matches
+            .value_of(options::WIDTH)
+            .expect(err_msg)
+            .parse()
+            .map_err(PtxError::ParseError)?;
     }
     if matches.is_present(options::GAP_SIZE) {
-        let gap_str = matches
+        config.gap_size = matches
             .value_of(options::GAP_SIZE)
             .expect(err_msg)
-            .to_string();
-        config.gap_size = crash_if_err!(1, (&gap_str).parse::<usize>());
+            .parse()
+            .map_err(PtxError::ParseError)?;
     }
     if matches.is_present(options::FORMAT_ROFF) {
         config.format = OutFormat::Roff;
@@ -195,7 +232,7 @@ fn get_config(matches: &clap::ArgMatches) -> Config {
     if matches.is_present(options::FORMAT_TEX) {
         config.format = OutFormat::Tex;
     }
-    config
+    Ok(config)
 }
 
 struct FileContent {
@@ -206,7 +243,7 @@ struct FileContent {
 
 type FileMap = HashMap<String, FileContent>;
 
-fn read_input(input_files: &[String], config: &Config) -> FileMap {
+fn read_input(input_files: &[String], config: &Config) -> std::io::Result<FileMap> {
     let mut file_map: FileMap = HashMap::new();
     let mut files = Vec::new();
     if input_files.is_empty() {
@@ -223,10 +260,10 @@ fn read_input(input_files: &[String], config: &Config) -> FileMap {
         let reader: BufReader<Box<dyn Read>> = BufReader::new(if filename == "-" {
             Box::new(stdin())
         } else {
-            let file = crash_if_err!(1, File::open(filename));
+            let file = File::open(filename)?;
             Box::new(file)
         });
-        let lines: Vec<String> = reader.lines().map(|x| crash_if_err!(1, x)).collect();
+        let lines: Vec<String> = reader.lines().collect::<std::io::Result<Vec<String>>>()?;
 
         // Indexing UTF-8 string requires walking from the beginning, which can hurts performance badly when the line is long.
         // Since we will be jumping around the line a lot, we dump the content into a Vec<char>, which can be indexed in constant time.
@@ -240,9 +277,9 @@ fn read_input(input_files: &[String], config: &Config) -> FileMap {
                 offset,
             },
         );
-        offset += size
+        offset += size;
     }
-    file_map
+    Ok(file_map)
 }
 
 /// Go through every lines in the input files and record each match occurrence as a `WordRef`.
@@ -292,7 +329,11 @@ fn create_word_set(config: &Config, filter: &WordFilter, file_map: &FileMap) ->
 
 fn get_reference(config: &Config, word_ref: &WordRef, line: &str, context_reg: &Regex) -> String {
     if config.auto_ref {
-        format!("{}:{}", word_ref.filename, word_ref.local_line_nr + 1)
+        format!(
+            "{}:{}",
+            word_ref.filename.maybe_quote(),
+            word_ref.local_line_nr + 1
+        )
     } else if config.input_ref {
         let (beg, end) = match context_reg.find(line) {
             Some(x) => (x.start(), x.end()),
@@ -521,7 +562,7 @@ fn format_tex_line(
 }
 
 fn format_roff_field(s: &str) -> String {
-    s.replace("\"", "\"\"")
+    s.replace('\"', "\"\"")
 }
 
 fn format_roff_line(
@@ -566,11 +607,11 @@ fn write_traditional_output(
     file_map: &FileMap,
     words: &BTreeSet<WordRef>,
     output_filename: &str,
-) {
+) -> UResult<()> {
     let mut writer: BufWriter<Box<dyn Write>> = BufWriter::new(if output_filename == "-" {
         Box::new(stdout())
     } else {
-        let file = crash_if_err!(1, File::create(output_filename));
+        let file = File::create(output_filename).map_err_context(String::new)?;
         Box::new(file)
     });
 
@@ -606,10 +647,13 @@ fn write_traditional_output(
                 &chars_lines[word_ref.local_line_nr],
                 &reference,
             ),
-            OutFormat::Dumb => crash!(1, "There is no dumb format with GNU extensions disabled"),
+            OutFormat::Dumb => {
+                return Err(PtxError::DumbFormat.into());
+            }
         };
-        crash_if_err!(1, writeln!(writer, "{}", output_line));
+        writeln!(writer, "{}", output_line).map_err_context(String::new)?;
     }
+    Ok(())
 }
 
 mod options {
@@ -632,7 +676,8 @@ mod options {
     pub static WIDTH: &str = "width";
 }
 
-pub fn uumain(args: impl uucore::Args) -> i32 {
+#[uucore::main]
+pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let args = args
         .collect_str(InvalidEncodingHandling::Ignore)
         .accept_any();
@@ -640,144 +685,149 @@ pub fn uumain(args: impl uucore::Args) -> i32 {
     // let mut opts = Options::new();
     let matches = uu_app().get_matches_from(args);
 
-    let input_files: Vec<String> = match &matches.values_of(options::FILE) {
+    let mut input_files: Vec<String> = match &matches.values_of(options::FILE) {
         Some(v) => v.clone().map(|v| v.to_owned()).collect(),
         None => vec!["-".to_string()],
     };
 
-    let config = get_config(&matches);
-    let word_filter = WordFilter::new(&matches, &config);
-    let file_map = read_input(&input_files, &config);
+    let config = get_config(&matches)?;
+    let word_filter = WordFilter::new(&matches, &config)?;
+    let file_map = read_input(&input_files, &config).map_err_context(String::new)?;
     let word_set = create_word_set(&config, &word_filter, &file_map);
-    let output_file = if !config.gnu_ext && matches.args.len() == 2 {
-        matches.value_of(options::FILE).unwrap_or("-").to_string()
+    let output_file = if !config.gnu_ext && input_files.len() == 2 {
+        input_files.pop().unwrap()
     } else {
-        "-".to_owned()
+        "-".to_string()
     };
-    write_traditional_output(&config, &file_map, &word_set, &output_file);
-    0
+    write_traditional_output(&config, &file_map, &word_set, &output_file)
 }
 
-pub fn uu_app() -> App<'static, 'static> {
-    App::new(executable!())
+pub fn uu_app<'a>() -> Command<'a> {
+    Command::new(uucore::util_name())
         .name(NAME)
+        .about(ABOUT)
         .version(crate_version!())
-        .usage(BRIEF)
-        .arg(Arg::with_name(options::FILE).hidden(true).multiple(true))
+        .override_usage(format_usage(USAGE))
+        .infer_long_args(true)
         .arg(
-            Arg::with_name(options::AUTO_REFERENCE)
-                .short("A")
+            Arg::new(options::FILE)
+                .hide(true)
+                .multiple_occurrences(true),
+        )
+        .arg(
+            Arg::new(options::AUTO_REFERENCE)
+                .short('A')
                 .long(options::AUTO_REFERENCE)
                 .help("output automatically generated references")
                 .takes_value(false),
         )
         .arg(
-            Arg::with_name(options::TRADITIONAL)
-                .short("G")
+            Arg::new(options::TRADITIONAL)
+                .short('G')
                 .long(options::TRADITIONAL)
                 .help("behave more like System V 'ptx'"),
         )
         .arg(
-            Arg::with_name(options::FLAG_TRUNCATION)
-                .short("F")
+            Arg::new(options::FLAG_TRUNCATION)
+                .short('F')
                 .long(options::FLAG_TRUNCATION)
                 .help("use STRING for flagging line truncations")
                 .value_name("STRING")
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(options::MACRO_NAME)
-                .short("M")
+            Arg::new(options::MACRO_NAME)
+                .short('M')
                 .long(options::MACRO_NAME)
                 .help("macro name to use instead of 'xx'")
                 .value_name("STRING")
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(options::FORMAT_ROFF)
-                .short("O")
+            Arg::new(options::FORMAT_ROFF)
+                .short('O')
                 .long(options::FORMAT_ROFF)
                 .help("generate output as roff directives"),
         )
         .arg(
-            Arg::with_name(options::RIGHT_SIDE_REFS)
-                .short("R")
+            Arg::new(options::RIGHT_SIDE_REFS)
+                .short('R')
                 .long(options::RIGHT_SIDE_REFS)
                 .help("put references at right, not counted in -w")
                 .takes_value(false),
         )
         .arg(
-            Arg::with_name(options::SENTENCE_REGEXP)
-                .short("S")
+            Arg::new(options::SENTENCE_REGEXP)
+                .short('S')
                 .long(options::SENTENCE_REGEXP)
                 .help("for end of lines or end of sentences")
                 .value_name("REGEXP")
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(options::FORMAT_TEX)
-                .short("T")
+            Arg::new(options::FORMAT_TEX)
+                .short('T')
                 .long(options::FORMAT_TEX)
                 .help("generate output as TeX directives"),
         )
         .arg(
-            Arg::with_name(options::WORD_REGEXP)
-                .short("W")
+            Arg::new(options::WORD_REGEXP)
+                .short('W')
                 .long(options::WORD_REGEXP)
                 .help("use REGEXP to match each keyword")
                 .value_name("REGEXP")
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(options::BREAK_FILE)
-                .short("b")
+            Arg::new(options::BREAK_FILE)
+                .short('b')
                 .long(options::BREAK_FILE)
                 .help("word break characters in this FILE")
                 .value_name("FILE")
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(options::IGNORE_CASE)
-                .short("f")
+            Arg::new(options::IGNORE_CASE)
+                .short('f')
                 .long(options::IGNORE_CASE)
                 .help("fold lower case to upper case for sorting")
                 .takes_value(false),
         )
         .arg(
-            Arg::with_name(options::GAP_SIZE)
-                .short("g")
+            Arg::new(options::GAP_SIZE)
+                .short('g')
                 .long(options::GAP_SIZE)
                 .help("gap size in columns between output fields")
                 .value_name("NUMBER")
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(options::IGNORE_FILE)
-                .short("i")
+            Arg::new(options::IGNORE_FILE)
+                .short('i')
                 .long(options::IGNORE_FILE)
                 .help("read ignore word list from FILE")
                 .value_name("FILE")
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(options::ONLY_FILE)
-                .short("o")
+            Arg::new(options::ONLY_FILE)
+                .short('o')
                 .long(options::ONLY_FILE)
                 .help("read only word list from this FILE")
                 .value_name("FILE")
                 .takes_value(true),
         )
         .arg(
-            Arg::with_name(options::REFERENCES)
-                .short("r")
+            Arg::new(options::REFERENCES)
+                .short('r')
                 .long(options::REFERENCES)
                 .help("first field of each line is a reference")
                 .value_name("FILE")
                 .takes_value(false),
         )
         .arg(
-            Arg::with_name(options::WIDTH)
-                .short("w")
+            Arg::new(options::WIDTH)
+                .short('w')
                 .long(options::WIDTH)
                 .help("output width in columns, reference excluded")
                 .value_name("NUMBER")
