@@ -39,10 +39,10 @@ struct WithEnvVarSet {
 }
 impl WithEnvVarSet {
     /// Save previous value assigned to key, set key=value
-    fn new(key: &str, value: &str) -> WithEnvVarSet {
+    fn new(key: &str, value: &str) -> Self {
         let previous_env_value = env::var(key);
         env::set_var(key, value);
-        WithEnvVarSet {
+        Self {
             _previous_var_key: String::from(key),
             _previous_var_value: previous_env_value,
         }
@@ -55,7 +55,7 @@ impl Drop for WithEnvVarSet {
         if let Ok(ref prev_value) = self._previous_var_value {
             env::set_var(&self._previous_var_key, &prev_value);
         } else {
-            env::remove_var(&self._previous_var_key)
+            env::remove_var(&self._previous_var_key);
         }
     }
 }
@@ -66,7 +66,7 @@ impl FilterWriter {
     ///
     /// * `command` - The shell command to execute
     /// * `filepath` - Path of the output file (forwarded to command as $FILE)
-    fn new(command: &str, filepath: &str) -> FilterWriter {
+    fn new(command: &str, filepath: &str) -> Self {
         // set $FILE, save previous value (if there was one)
         let _with_env_var_set = WithEnvVarSet::new("FILE", filepath);
 
@@ -78,7 +78,7 @@ impl FilterWriter {
                 .spawn()
                 .expect("Couldn't spawn filter command");
 
-        FilterWriter { shell_process }
+        Self { shell_process }
     }
 }
 
