@@ -8,7 +8,7 @@
 // spell-checker:ignore (ToDO) sbytes slen dlen memmem memmap Mmap mmap SIGBUS
 mod error;
 
-use clap::{crate_version, App, Arg};
+use clap::{crate_version, Arg, Command};
 use memchr::memmem;
 use memmap2::Mmap;
 use std::io::{stdin, stdout, BufWriter, Read, Write};
@@ -19,13 +19,13 @@ use std::{
 use uucore::display::Quotable;
 use uucore::error::UError;
 use uucore::error::UResult;
-use uucore::show;
 use uucore::InvalidEncodingHandling;
+use uucore::{format_usage, show};
 
 use crate::error::TacError;
 
 static NAME: &str = "tac";
-static USAGE: &str = "[OPTION]... [FILE]...";
+static USAGE: &str = "{} [OPTION]... [FILE]...";
 static SUMMARY: &str = "Write each file to standard output, last line first.";
 
 mod options {
@@ -35,7 +35,7 @@ mod options {
     pub static FILE: &str = "file";
 }
 
-#[uucore_procs::gen_uumain]
+#[uucore::main]
 pub fn uumain(args: impl uucore::Args) -> UResult<()> {
     let args = args
         .collect_str(InvalidEncodingHandling::ConvertLossy)
@@ -57,37 +57,42 @@ pub fn uumain(args: impl uucore::Args) -> UResult<()> {
         None => vec!["-"],
     };
 
-    tac(files, before, regex, separator)
+    tac(&files, before, regex, separator)
 }
 
-pub fn uu_app() -> App<'static, 'static> {
-    App::new(uucore::util_name())
+pub fn uu_app<'a>() -> Command<'a> {
+    Command::new(uucore::util_name())
         .name(NAME)
         .version(crate_version!())
-        .usage(USAGE)
+        .override_usage(format_usage(USAGE))
         .about(SUMMARY)
+        .infer_long_args(true)
         .arg(
-            Arg::with_name(options::BEFORE)
-                .short("b")
+            Arg::new(options::BEFORE)
+                .short('b')
                 .long(options::BEFORE)
                 .help("attach the separator before instead of after")
                 .takes_value(false),
         )
         .arg(
-            Arg::with_name(options::REGEX)
-                .short("r")
+            Arg::new(options::REGEX)
+                .short('r')
                 .long(options::REGEX)
                 .help("interpret the sequence as a regular expression")
                 .takes_value(false),
         )
         .arg(
-            Arg::with_name(options::SEPARATOR)
-                .short("s")
+            Arg::new(options::SEPARATOR)
+                .short('s')
                 .long(options::SEPARATOR)
                 .help("use STRING as the separator instead of newline")
                 .takes_value(true),
         )
-        .arg(Arg::with_name(options::FILE).hidden(true).multiple(true))
+        .arg(
+            Arg::new(options::FILE)
+                .hide(true)
+                .multiple_occurrences(true),
+        )
 }
 
 /// Print lines of a buffer in reverse, with line separator given as a regex.
@@ -218,7 +223,7 @@ fn buffer_tac(data: &[u8], before: bool, separator: &str) -> std::io::Result<()>
     Ok(())
 }
 
-fn tac(filenames: Vec<&str>, before: bool, regex: bool, separator: &str) -> UResult<()> {
+fn tac(filenames: &[&str], before: bool, regex: bool, separator: &str) -> UResult<()> {
     // Compile the regular expression pattern if it is provided.
     let maybe_pattern = if regex {
         match regex::bytes::Regex::new(separator) {
@@ -229,7 +234,7 @@ fn tac(filenames: Vec<&str>, before: bool, regex: bool, separator: &str) -> URes
         None
     };
 
-    for &filename in &filenames {
+    for &filename in filenames {
         let mmap;
         let buf;
 
