@@ -15,17 +15,21 @@ use count_fast::{count_bytes_and_lines_fast, count_bytes_fast};
 use countable::WordCountable;
 use unicode_width::UnicodeWidthChar;
 use utf8::{BufReadDecoder, BufReadDecoderError};
+use uucore::format_usage;
 use word_count::{TitledWordCount, WordCount};
 
-use clap::{crate_version, App, Arg, ArgMatches};
+use clap::{crate_version, Arg, ArgMatches, Command};
 
 use std::cmp::max;
+use std::error::Error;
+use std::ffi::OsStr;
+use std::fmt::Display;
 use std::fs::{self, File};
-use std::io::{self, Write};
+use std::io::{self, Read, Write};
 use std::path::{Path, PathBuf};
 
 use uucore::display::{Quotable, Quoted};
-use uucore::error::{UResult, USimpleError};
+use uucore::error::{UError, UResult, USimpleError};
 
 /// The minimum character width for formatting counts when reading from stdin.
 const MINIMUM_WIDTH: usize = 7;
@@ -39,8 +43,8 @@ struct Settings {
 }
 
 impl Settings {
-    fn new(matches: &ArgMatches) -> Settings {
-        let settings = Settings {
+    fn new(matches: &ArgMatches) -> Self {
+        let settings = Self {
             show_bytes: matches.is_present(options::BYTES),
             show_chars: matches.is_present(options::CHAR),
             show_lines: matches.is_present(options::LINES),
@@ -57,7 +61,7 @@ impl Settings {
             return settings;
         }
 
-        Settings {
+        Self {
             show_bytes: true,
             show_chars: false,
             show_lines: true,
@@ -78,25 +82,20 @@ impl Settings {
 }
 
 static ABOUT: &str = "Display newline, word, and byte counts for each FILE, and a total line if
-more than one FILE is specified.";
+more than one FILE is specified. With no FILE, or when FILE is -, read standard input.";
+const USAGE: &str = "{} [OPTION]... [FILE]...";
 
 pub mod options {
     pub static BYTES: &str = "bytes";
     pub static CHAR: &str = "chars";
+    pub static FILES0_FROM: &str = "files0-from";
     pub static LINES: &str = "lines";
     pub static MAX_LINE_LENGTH: &str = "max-line-length";
     pub static WORDS: &str = "words";
 }
 
 static ARG_FILES: &str = "files";
-
-fn usage() -> String {
-    format!(
-        "{0} [OPTION]... [FILE]...
- With no FILE, or when FILE is -, read standard input.",
-        uucore::execution_phrase()
-    )
-}
+static STDIN_REPR: &str = "-";
 
 enum StdinKind {
     /// Stdin specified on command-line with "-".
@@ -115,12 +114,22 @@ enum Input {
     Stdin(StdinKind),
 }
 
+impl From<&OsStr> for Input {
+    fn from(input: &OsStr) -> Self {
+        if input == STDIN_REPR {
+            Self::Stdin(StdinKind::Explicit)
+        } else {
+            Self::Path(input.into())
+        }
+    }
+}
+
 impl Input {
     /// Converts input to title that appears in stats.
     fn to_title(&self) -> Option<&Path> {
         match self {
             Input::Path(path) => Some(path),
-            Input::Stdin(StdinKind::Explicit) => Some("-".as_ref()),
+            Input::Stdin(StdinKind::Explicit) => Some(STDIN_REPR.as_ref()),
             Input::Stdin(StdinKind::Implicit) => None,
         }
     }
@@ -133,70 +142,141 @@ impl Input {
     }
 }
 
-#[uucore_procs::gen_uumain]
-pub fn uumain(args: impl uucore::Args) -> UResult<()> {
-    let usage = usage();
+#[derive(Debug)]
+enum WcError {
+    FilesDisabled(String),
+    StdinReprNotAllowed(String),
+}
 
-    let matches = uu_app().usage(&usage[..]).get_matches_from(args);
-
-    let mut inputs: Vec<Input> = matches
-        .values_of_os(ARG_FILES)
-        .map(|v| {
-            v.map(|i| {
-                if i == "-" {
-                    Input::Stdin(StdinKind::Explicit)
-                } else {
-                    Input::Path(i.into())
-                }
-            })
-            .collect()
-        })
-        .unwrap_or_default();
-
-    if inputs.is_empty() {
-        inputs.push(Input::Stdin(StdinKind::Implicit));
+impl UError for WcError {
+    fn code(&self) -> i32 {
+        match self {
+            WcError::FilesDisabled(_) | WcError::StdinReprNotAllowed(_) => 1,
+        }
     }
 
+    fn usage(&self) -> bool {
+        matches!(self, WcError::FilesDisabled(_))
+    }
+}
+
+impl Error for WcError {}
+
+impl Display for WcError {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        match self {
+            WcError::FilesDisabled(message) | WcError::StdinReprNotAllowed(message) => {
+                write!(f, "{}", message)
+            }
+        }
+    }
+}
+
+#[uucore::main]
+pub fn uumain(args: impl uucore::Args) -> UResult<()> {
+    let matches = uu_app().get_matches_from(args);
+
+    let inputs = inputs(&matches)?;
+
     let settings = Settings::new(&matches);
 
-    wc(inputs, &settings)
+    wc(&inputs, &settings)
 }
 
-pub fn uu_app() -> App<'static, 'static> {
-    App::new(uucore::util_name())
+pub fn uu_app<'a>() -> Command<'a> {
+    Command::new(uucore::util_name())
         .version(crate_version!())
         .about(ABOUT)
+        .override_usage(format_usage(USAGE))
+        .infer_long_args(true)
         .arg(
-            Arg::with_name(options::BYTES)
-                .short("c")
+            Arg::new(options::BYTES)
+                .short('c')
                 .long(options::BYTES)
                 .help("print the byte counts"),
         )
         .arg(
-            Arg::with_name(options::CHAR)
-                .short("m")
+            Arg::new(options::CHAR)
+                .short('m')
                 .long(options::CHAR)
                 .help("print the character counts"),
         )
         .arg(
-            Arg::with_name(options::LINES)
-                .short("l")
+            Arg::new(options::FILES0_FROM)
+                .long(options::FILES0_FROM)
+                .takes_value(true)
+                .value_name("F")
+                .help(
+                    "read input from the files specified by
+    NUL-terminated names in file F;
+    If F is - then read names from standard input",
+                ),
+        )
+        .arg(
+            Arg::new(options::LINES)
+                .short('l')
                 .long(options::LINES)
                 .help("print the newline counts"),
         )
         .arg(
-            Arg::with_name(options::MAX_LINE_LENGTH)
-                .short("L")
+            Arg::new(options::MAX_LINE_LENGTH)
+                .short('L')
                 .long(options::MAX_LINE_LENGTH)
                 .help("print the length of the longest line"),
         )
         .arg(
-            Arg::with_name(options::WORDS)
-                .short("w")
+            Arg::new(options::WORDS)
+                .short('w')
                 .long(options::WORDS)
                 .help("print the word counts"),
         )
-        .arg(Arg::with_name(ARG_FILES).multiple(true).takes_value(true))
+        .arg(
+            Arg::new(ARG_FILES)
+                .multiple_occurrences(true)
+                .takes_value(true)
+                .allow_invalid_utf8(true),
+        )
+}
+
+fn inputs(matches: &ArgMatches) -> UResult<Vec<Input>> {
+    match matches.values_of_os(ARG_FILES) {
+        Some(os_values) => {
+            if matches.is_present(options::FILES0_FROM) {
+                return Err(WcError::FilesDisabled(
+                    "file operands cannot be combined with --files0-from".into(),
+                )
+                .into());
+            }
+
+            Ok(os_values.map(Input::from).collect())
+        }
+        None => match matches.value_of(options::FILES0_FROM) {
+            Some(files_0_from) => create_paths_from_files0(files_0_from),
+            None => Ok(vec![Input::Stdin(StdinKind::Implicit)]),
+        },
+    }
+}
+
+fn create_paths_from_files0(files_0_from: &str) -> UResult<Vec<Input>> {
+    let mut paths = String::new();
+    let read_from_stdin = files_0_from == STDIN_REPR;
+
+    if read_from_stdin {
+        io::stdin().lock().read_to_string(&mut paths)?;
+    } else {
+        File::open(files_0_from)?.read_to_string(&mut paths)?;
+    }
+
+    let paths: Vec<&str> = paths.split_terminator('\0').collect();
+
+    if read_from_stdin && paths.contains(&STDIN_REPR) {
+        return Err(WcError::StdinReprNotAllowed(
+            "when reading file names from stdin, no file name of '-' allowed".into(),
+        )
+        .into());
+    }
+
+    Ok(paths.iter().map(OsStr::new).map(Input::from).collect())
 }
 
 fn word_count_from_reader<T: WordCountable>(
@@ -247,12 +327,7 @@ fn word_count_from_reader<T: WordCountable>(
                     }
                     if settings.show_max_line_length {
                         match ch {
-                            '\n' => {
-                                total.max_line_length = max(current_len, total.max_line_length);
-                                current_len = 0;
-                            }
-                            // '\x0c' = '\f'
-                            '\r' | '\x0c' => {
+                            '\n' | '\r' | '\x0c' => {
                                 total.max_line_length = max(current_len, total.max_line_length);
                                 current_len = 0;
                             }
@@ -324,46 +399,6 @@ fn word_count_from_input(input: &Input, settings: &Settings) -> CountResult {
     }
 }
 
-/// Compute the number of digits needed to represent any count for this input.
-///
-/// If `input` is [`Input::Stdin`], then this function returns
-/// [`MINIMUM_WIDTH`]. Otherwise, if metadata could not be read from
-/// `input` then this function returns 1.
-///
-/// # Errors
-///
-/// This function will return an error if `input` is a [`Input::Path`]
-/// and there is a problem accessing the metadata of the given `input`.
-///
-/// # Examples
-///
-/// A [`Input::Stdin`] gets a default minimum width:
-///
-/// ```rust,ignore
-/// let input = Input::Stdin(StdinKind::Explicit);
-/// assert_eq!(7, digit_width(input));
-/// ```
-fn digit_width(input: &Input) -> io::Result<usize> {
-    match input {
-        Input::Stdin(_) => Ok(MINIMUM_WIDTH),
-        Input::Path(filename) => {
-            let path = Path::new(filename);
-            let metadata = fs::metadata(path)?;
-            if metadata.is_file() {
-                // TODO We are now computing the number of bytes in a file
-                // twice: once here and once in `WordCount::from_line()` (or
-                // in `count_bytes_fast()` if that function is called
-                // instead). See GitHub issue #2201.
-                let num_bytes = metadata.len();
-                let num_digits = num_bytes.to_string().len();
-                Ok(num_digits)
-            } else {
-                Ok(MINIMUM_WIDTH)
-            }
-        }
-    }
-}
-
 /// Compute the number of digits needed to represent all counts in all inputs.
 ///
 /// `inputs` may include zero or more [`Input::Stdin`] entries, each of
@@ -371,58 +406,51 @@ fn digit_width(input: &Input) -> io::Result<usize> {
 /// entry causes this function to return a width that is at least
 /// [`MINIMUM_WIDTH`].
 ///
-/// If `input` is empty, then this function returns 1. If file metadata
-/// could not be read from any of the [`Input::Path`] inputs and there
-/// are no [`Input::Stdin`] inputs, then this function returns 1.
+/// If `input` is empty, or if only one number needs to be printed (for just
+/// one file) then this function is optimized to return 1 without making any
+/// calls to get file metadata.
 ///
-/// If there is a problem accessing the metadata, this function will
-/// silently ignore the error and assume that the number of digits
-/// needed to display the counts for that file is 1.
+/// If file metadata could not be read from any of the [`Input::Path`] input,
+/// that input does not affect number width computation
 ///
-/// # Examples
-///
-/// An empty slice implies a width of 1:
-///
-/// ```rust,ignore
-/// assert_eq!(1, max_width(&vec![]));
-/// ```
-///
-/// The presence of [`Input::Stdin`] implies a minimum width:
-///
-/// ```rust,ignore
-/// let inputs = vec![Input::Stdin(StdinKind::Explicit)];
-/// assert_eq!(7, max_width(&inputs));
-/// ```
-fn max_width(inputs: &[Input]) -> usize {
-    let mut result = 1;
+/// Otherwise, the file sizes in the file metadata are summed and the number of
+/// digits in that total size is returned as the number width
+fn compute_number_width(inputs: &[Input], settings: &Settings) -> usize {
+    if inputs.is_empty() || (inputs.len() == 1 && settings.number_enabled() == 1) {
+        return 1;
+    }
+
+    let mut minimum_width = 1;
+    let mut total = 0;
+
     for input in inputs {
-        if let Ok(n) = digit_width(input) {
-            result = result.max(n);
+        match input {
+            Input::Stdin(_) => {
+                minimum_width = MINIMUM_WIDTH;
+            }
+            Input::Path(path) => {
+                if let Ok(meta) = fs::metadata(path) {
+                    if meta.is_file() {
+                        total += meta.len();
+                    } else {
+                        minimum_width = MINIMUM_WIDTH;
+                    }
+                }
+            }
         }
     }
-    result
+
+    max(minimum_width, total.to_string().len())
 }
 
-fn wc(inputs: Vec<Input>, settings: &Settings) -> UResult<()> {
-    // Compute the width, in digits, to use when formatting counts.
-    //
-    // The width is the number of digits needed to print the number of
-    // bytes in the largest file. This is true regardless of whether
-    // the `settings` indicate that the bytes will be displayed.
-    //
-    // If we only need to display a single number, set this to 0 to
-    // prevent leading spaces.
-    let max_width = if settings.number_enabled() <= 1 {
-        0
-    } else {
-        max_width(&inputs)
-    };
+fn wc(inputs: &[Input], settings: &Settings) -> UResult<()> {
+    let number_width = compute_number_width(inputs, settings);
 
     let mut total_word_count = WordCount::default();
 
     let num_inputs = inputs.len();
 
-    for input in &inputs {
+    for input in inputs {
         let word_count = match word_count_from_input(input, settings) {
             CountResult::Success(word_count) => word_count,
             CountResult::Interrupted(word_count, error) => {
@@ -442,7 +470,7 @@ fn wc(inputs: Vec<Input>, settings: &Settings) -> UResult<()> {
         };
         total_word_count += word_count;
         let result = word_count.with_title(input.to_title());
-        if let Err(err) = print_stats(settings, &result, max_width) {
+        if let Err(err) = print_stats(settings, &result, number_width) {
             show!(USimpleError::new(
                 1,
                 format!(
@@ -459,7 +487,7 @@ fn wc(inputs: Vec<Input>, settings: &Settings) -> UResult<()> {
 
     if num_inputs > 1 {
         let total_result = total_word_count.with_title(Some("total".as_ref()));
-        if let Err(err) = print_stats(settings, &total_result, max_width) {
+        if let Err(err) = print_stats(settings, &total_result, number_width) {
             show!(USimpleError::new(
                 1,
                 format!("failed to print total: {}", err)
@@ -472,56 +500,32 @@ fn wc(inputs: Vec<Input>, settings: &Settings) -> UResult<()> {
     Ok(())
 }
 
-fn print_stats(settings: &Settings, result: &TitledWordCount, min_width: usize) -> io::Result<()> {
-    let stdout = io::stdout();
-    let mut stdout_lock = stdout.lock();
-
-    let mut is_first: bool = true;
+fn print_stats(
+    settings: &Settings,
+    result: &TitledWordCount,
+    number_width: usize,
+) -> io::Result<()> {
+    let mut columns = Vec::new();
 
     if settings.show_lines {
-        if !is_first {
-            write!(stdout_lock, " ")?;
-        }
-        write!(stdout_lock, "{:1$}", result.count.lines, min_width)?;
-        is_first = false;
+        columns.push(format!("{:1$}", result.count.lines, number_width));
     }
     if settings.show_words {
-        if !is_first {
-            write!(stdout_lock, " ")?;
-        }
-        write!(stdout_lock, "{:1$}", result.count.words, min_width)?;
-        is_first = false;
+        columns.push(format!("{:1$}", result.count.words, number_width));
     }
     if settings.show_chars {
-        if !is_first {
-            write!(stdout_lock, " ")?;
-        }
-        write!(stdout_lock, "{:1$}", result.count.chars, min_width)?;
-        is_first = false;
+        columns.push(format!("{:1$}", result.count.chars, number_width));
     }
     if settings.show_bytes {
-        if !is_first {
-            write!(stdout_lock, " ")?;
-        }
-        write!(stdout_lock, "{:1$}", result.count.bytes, min_width)?;
-        is_first = false;
+        columns.push(format!("{:1$}", result.count.bytes, number_width));
     }
     if settings.show_max_line_length {
-        if !is_first {
-            write!(stdout_lock, " ")?;
-        }
-        write!(
-            stdout_lock,
-            "{:1$}",
-            result.count.max_line_length, min_width
-        )?;
+        columns.push(format!("{:1$}", result.count.max_line_length, number_width));
     }
 
     if let Some(title) = result.title {
-        writeln!(stdout_lock, " {}", title.maybe_quote())?;
-    } else {
-        writeln!(stdout_lock)?;
+        columns.push(title.maybe_quote().to_string());
     }
 
-    Ok(())
+    writeln!(io::stdout().lock(), "{}", columns.join(" "))
 }
