@@ -7,6 +7,8 @@
 // For the full copyright and license information, please view the LICENSE file
 // that was distributed with this source code.
 
+//! Set of functions to manage file systems
+
 // spell-checker:ignore (arch) bitrig ; (fs) cifs smbfs
 
 extern crate time;
@@ -24,16 +26,12 @@ const MAX_PATH: usize = 266;
 static EXIT_ERR: i32 = 1;
 
 #[cfg(windows)]
-use std::ffi::OsString;
+use std::ffi::OsStr;
 #[cfg(windows)]
 use std::os::windows::ffi::OsStrExt;
 #[cfg(windows)]
-use std::os::windows::ffi::OsStringExt;
-#[cfg(windows)]
 use winapi::shared::minwindef::DWORD;
 #[cfg(windows)]
-use winapi::um::errhandlingapi::GetLastError;
-#[cfg(windows)]
 use winapi::um::fileapi::GetDiskFreeSpaceW;
 #[cfg(windows)]
 use winapi::um::fileapi::{
@@ -45,11 +43,12 @@ use winapi::um::handleapi::INVALID_HANDLE_VALUE;
 #[cfg(windows)]
 use winapi::um::winbase::DRIVE_REMOTE;
 
+// Warning: the pointer has to be used *immediately* or the Vec
+// it points to will be dropped!
 #[cfg(windows)]
 macro_rules! String2LPWSTR {
     ($str: expr) => {
-        OsString::from($str.clone())
-            .as_os_str()
+        OsStr::new(&$str)
             .encode_wide()
             .chain(Some(0))
             .collect::<Vec<u16>>()
@@ -60,10 +59,8 @@ macro_rules! String2LPWSTR {
 #[cfg(windows)]
 #[allow(non_snake_case)]
 fn LPWSTR2String(buf: &[u16]) -> String {
-    let len = unsafe { libc::wcslen(buf.as_ptr()) };
-    OsString::from_wide(&buf[..len as usize])
-        .into_string()
-        .unwrap()
+    let len = buf.iter().position(|&n| n == 0).unwrap();
+    String::from_utf16(&buf[..len]).unwrap()
 }
 
 use self::time::Timespec;
@@ -73,9 +70,16 @@ use libc::{
 };
 use std::borrow::Cow;
 use std::convert::{AsRef, From};
-#[cfg(unix)]
+#[cfg(any(
+    target_vendor = "apple",
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd",
+    target_os = "linux"
+))]
+use std::ffi::CStr;
+#[cfg(not(windows))]
 use std::ffi::CString;
-#[cfg(unix)]
 use std::io::Error as IOError;
 #[cfg(unix)]
 use std::mem;
@@ -86,15 +90,17 @@ use std::time::UNIX_EPOCH;
     target_os = "linux",
     target_vendor = "apple",
     target_os = "android",
-    target_os = "freebsd"
+    target_os = "freebsd",
+    target_os = "openbsd"
 ))]
 pub use libc::statfs as StatFs;
 #[cfg(any(
-    target_os = "openbsd",
     target_os = "netbsd",
-    target_os = "openbsd",
     target_os = "bitrig",
-    target_os = "dragonfly"
+    target_os = "dragonfly",
+    target_os = "illumos",
+    target_os = "solaris",
+    target_os = "redox"
 ))]
 pub use libc::statvfs as StatFs;
 
@@ -102,14 +108,16 @@ pub use libc::statvfs as StatFs;
     target_os = "linux",
     target_vendor = "apple",
     target_os = "android",
-    target_os = "freebsd"
+    target_os = "freebsd",
+    target_os = "openbsd",
+    target_os = "redox"
 ))]
 pub use libc::statfs as statfs_fn;
 #[cfg(any(
-    target_os = "openbsd",
     target_os = "netbsd",
-    target_os = "openbsd",
     target_os = "bitrig",
+    target_os = "illumos",
+    target_os = "solaris",
     target_os = "dragonfly"
 ))]
 pub use libc::statvfs as statfs_fn;
@@ -155,16 +163,14 @@ impl MountInfo {
     fn set_missing_fields(&mut self) {
         #[cfg(unix)]
         {
+            use std::os::unix::fs::MetadataExt;
             // We want to keep the dev_id on Windows
             // but set dev_id
-            let path = CString::new(self.mount_dir.clone()).unwrap();
-            unsafe {
-                let mut stat = mem::zeroed();
-                if libc::stat(path.as_ptr(), &mut stat) == 0 {
-                    self.dev_id = (stat.st_dev as i32).to_string();
-                } else {
-                    self.dev_id = "".to_string();
-                }
+            if let Ok(stat) = std::fs::metadata(&self.mount_dir) {
+                // Why do we cast this to i32?
+                self.dev_id = (stat.dev() as i32).to_string();
+            } else {
+                self.dev_id = "".to_string();
             }
         }
         // set MountInfo::dummy
@@ -203,16 +209,20 @@ impl MountInfo {
     }
 
     #[cfg(target_os = "linux")]
-    fn new(file_name: &str, raw: Vec<&str>) -> Option<MountInfo> {
+    fn new(file_name: &str, raw: &[&str]) -> Option<Self> {
         match file_name {
             // spell-checker:ignore (word) noatime
             // Format: 36 35 98:0 /mnt1 /mnt2 rw,noatime master:1 - ext3 /dev/root rw,errors=continue
             // "man proc" for more details
             LINUX_MOUNTINFO => {
-                let mut m = MountInfo {
+                const FIELDS_OFFSET: usize = 6;
+                let after_fields = raw[FIELDS_OFFSET..].iter().position(|c| *c == "-").unwrap()
+                    + FIELDS_OFFSET
+                    + 1;
+                let mut m = Self {
                     dev_id: "".to_string(),
-                    dev_name: raw[9].to_string(),
-                    fs_type: raw[8].to_string(),
+                    dev_name: raw[after_fields + 1].to_string(),
+                    fs_type: raw[after_fields].to_string(),
                     mount_root: raw[3].to_string(),
                     mount_dir: raw[4].to_string(),
                     mount_option: raw[5].to_string(),
@@ -223,7 +233,7 @@ impl MountInfo {
                 Some(m)
             }
             LINUX_MTAB => {
-                let mut m = MountInfo {
+                let mut m = Self {
                     dev_id: "".to_string(),
                     dev_name: raw[0].to_string(),
                     fs_type: raw[2].to_string(),
@@ -240,13 +250,12 @@ impl MountInfo {
         }
     }
     #[cfg(windows)]
-    fn new(mut volume_name: String) -> Option<MountInfo> {
+    fn new(mut volume_name: String) -> Option<Self> {
         let mut dev_name_buf = [0u16; MAX_PATH];
         volume_name.pop();
         unsafe {
             QueryDosDeviceW(
-                OsString::from(volume_name.clone())
-                    .as_os_str()
+                OsStr::new(&volume_name)
                     .encode_wide()
                     .chain(Some(0))
                     .skip(4)
@@ -292,7 +301,7 @@ impl MountInfo {
         } else {
             None
         };
-        let mut mn_info = MountInfo {
+        let mut mn_info = Self {
             dev_id: volume_name,
             dev_name,
             fs_type: fs_type.unwrap_or_else(|| "".to_string()),
@@ -307,12 +316,15 @@ impl MountInfo {
     }
 }
 
-#[cfg(any(target_vendor = "apple", target_os = "freebsd"))]
-use std::ffi::CStr;
-#[cfg(any(target_os = "freebsd", target_vendor = "apple"))]
+#[cfg(any(
+    target_os = "freebsd",
+    target_vendor = "apple",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
 impl From<StatFs> for MountInfo {
     fn from(statfs: StatFs) -> Self {
-        let mut info = MountInfo {
+        let mut info = Self {
             dev_id: "".to_string(),
             dev_name: unsafe {
                 // spell-checker:disable-next-line
@@ -342,9 +354,19 @@ impl From<StatFs> for MountInfo {
     }
 }
 
-#[cfg(any(target_os = "freebsd", target_vendor = "apple"))]
+#[cfg(any(
+    target_os = "freebsd",
+    target_vendor = "apple",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
 use libc::c_int;
-#[cfg(any(target_os = "freebsd", target_vendor = "apple"))]
+#[cfg(any(
+    target_os = "freebsd",
+    target_vendor = "apple",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
 extern "C" {
     #[cfg(all(target_vendor = "apple", target_arch = "x86_64"))]
     #[link_name = "getmntinfo$INODE64"] // spell-checker:disable-line
@@ -352,6 +374,8 @@ extern "C" {
 
     #[cfg(any(
         all(target_os = "freebsd"),
+        all(target_os = "netbsd"),
+        all(target_os = "openbsd"),
         all(target_vendor = "apple", target_arch = "aarch64")
     ))]
     #[link_name = "getmntinfo"] // spell-checker:disable-line
@@ -362,9 +386,20 @@ extern "C" {
 use std::fs::File;
 #[cfg(target_os = "linux")]
 use std::io::{BufRead, BufReader};
-#[cfg(any(target_vendor = "apple", target_os = "freebsd", target_os = "windows"))]
+#[cfg(any(
+    target_vendor = "apple",
+    target_os = "freebsd",
+    target_os = "windows",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
 use std::ptr;
-#[cfg(any(target_vendor = "apple", target_os = "freebsd"))]
+#[cfg(any(
+    target_vendor = "apple",
+    target_os = "freebsd",
+    target_os = "netbsd",
+    target_os = "openbsd"
+))]
 use std::slice;
 /// Read file system list.
 pub fn read_fs_list() -> Vec<MountInfo> {
@@ -380,11 +415,16 @@ pub fn read_fs_list() -> Vec<MountInfo> {
             .filter_map(|line| line.ok())
             .filter_map(|line| {
                 let raw_data = line.split_whitespace().collect::<Vec<&str>>();
-                MountInfo::new(file_name, raw_data)
+                MountInfo::new(file_name, &raw_data)
             })
             .collect::<Vec<_>>()
     }
-    #[cfg(any(target_os = "freebsd", target_vendor = "apple"))]
+    #[cfg(any(
+        target_os = "freebsd",
+        target_vendor = "apple",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    ))]
     {
         let mut mount_buffer_ptr: *mut StatFs = ptr::null_mut();
         let len = unsafe { get_mount_info(&mut mount_buffer_ptr, 1_i32) };
@@ -405,9 +445,11 @@ pub fn read_fs_list() -> Vec<MountInfo> {
             FindFirstVolumeW(volume_name_buf.as_mut_ptr(), volume_name_buf.len() as DWORD)
         };
         if INVALID_HANDLE_VALUE == find_handle {
-            crash!(EXIT_ERR, "FindFirstVolumeW failed: {}", unsafe {
-                GetLastError()
-            });
+            crash!(
+                EXIT_ERR,
+                "FindFirstVolumeW failed: {}",
+                IOError::last_os_error()
+            );
         }
         let mut mounts = Vec::<MountInfo>::new();
         loop {
@@ -426,8 +468,9 @@ pub fn read_fs_list() -> Vec<MountInfo> {
                     volume_name_buf.len() as DWORD,
                 )
             } {
-                let err = unsafe { GetLastError() };
-                if err != winapi::shared::winerror::ERROR_NO_MORE_FILES {
+                let err = IOError::last_os_error();
+                if err.raw_os_error() != Some(winapi::shared::winerror::ERROR_NO_MORE_FILES as i32)
+                {
                     crash!(EXIT_ERR, "FindNextVolumeW failed: {}", err);
                 }
                 break;
@@ -438,6 +481,11 @@ pub fn read_fs_list() -> Vec<MountInfo> {
         }
         mounts
     }
+    #[cfg(any(target_os = "redox", target_os = "illumos", target_os = "solaris"))]
+    {
+        // No method to read mounts, yet
+        Vec::new()
+    }
 }
 
 #[derive(Debug, Clone)]
@@ -453,9 +501,9 @@ pub struct FsUsage {
 
 impl FsUsage {
     #[cfg(unix)]
-    pub fn new(statvfs: StatFs) -> FsUsage {
+    pub fn new(statvfs: StatFs) -> Self {
         {
-            FsUsage {
+            Self {
                 blocksize: statvfs.f_bsize as u64, // or `statvfs.f_frsize` ?
                 blocks: statvfs.f_blocks as u64,
                 bfree: statvfs.f_bfree as u64,
@@ -467,7 +515,7 @@ impl FsUsage {
         }
     }
     #[cfg(not(unix))]
-    pub fn new(path: &Path) -> FsUsage {
+    pub fn new(path: &Path) -> Self {
         let mut root_path = [0u16; MAX_PATH];
         let success = unsafe {
             GetVolumePathNamesForVolumeNameW(
@@ -482,7 +530,7 @@ impl FsUsage {
             crash!(
                 EXIT_ERR,
                 "GetVolumePathNamesForVolumeNameW failed: {}",
-                unsafe { GetLastError() }
+                IOError::last_os_error()
             );
         }
 
@@ -502,13 +550,15 @@ impl FsUsage {
         };
         if 0 == success {
             // Fails in case of CD for example
-            //crash!(EXIT_ERR, "GetDiskFreeSpaceW failed: {}", unsafe {
-            //GetLastError()
-            //});
+            // crash!(
+            //     EXIT_ERR,
+            //     "GetDiskFreeSpaceW failed: {}",
+            //     IOError::last_os_error()
+            // );
         }
 
         let bytes_per_cluster = sectors_per_cluster as u64 * bytes_per_sector as u64;
-        FsUsage {
+        Self {
             // f_bsize      File system block size.
             blocksize: bytes_per_cluster as u64,
             // f_blocks - Total number of blocks on the file system, in units of f_frsize.
@@ -522,7 +572,7 @@ impl FsUsage {
             // Total number of file nodes (inodes) on the file system.
             files: 0, // Not available on windows
             // Total number of free file nodes (inodes).
-            ffree: 4096, // Meaningless on Windows
+            ffree: 0, // Meaningless on Windows
         }
     }
 }
@@ -575,12 +625,17 @@ impl FsMeta for StatFs {
     fn io_size(&self) -> u64 {
         self.f_frsize as u64
     }
-    #[cfg(any(target_vendor = "apple", target_os = "freebsd"))]
+    #[cfg(any(target_vendor = "apple", target_os = "freebsd", target_os = "netbsd"))]
     fn io_size(&self) -> u64 {
         self.f_iosize as u64
     }
     // XXX: dunno if this is right
-    #[cfg(not(any(target_vendor = "apple", target_os = "freebsd", target_os = "linux")))]
+    #[cfg(not(any(
+        target_vendor = "apple",
+        target_os = "freebsd",
+        target_os = "linux",
+        target_os = "netbsd"
+    )))]
     fn io_size(&self) -> u64 {
         self.f_bsize as u64
     }
@@ -591,13 +646,23 @@ impl FsMeta for StatFs {
     //
     // Solaris, Irix and POSIX have a system call statvfs(2) that returns a
     // struct statvfs, containing an  unsigned  long  f_fsid
-    #[cfg(any(target_vendor = "apple", target_os = "freebsd", target_os = "linux"))]
+    #[cfg(any(
+        target_vendor = "apple",
+        target_os = "freebsd",
+        target_os = "linux",
+        target_os = "openbsd"
+    ))]
     fn fsid(&self) -> u64 {
         let f_fsid: &[u32; 2] =
             unsafe { &*(&self.f_fsid as *const libc::fsid_t as *const [u32; 2]) };
         (u64::from(f_fsid[0])) << 32 | u64::from(f_fsid[1])
     }
-    #[cfg(not(any(target_vendor = "apple", target_os = "freebsd", target_os = "linux")))]
+    #[cfg(not(any(
+        target_vendor = "apple",
+        target_os = "freebsd",
+        target_os = "linux",
+        target_os = "openbsd"
+    )))]
     fn fsid(&self) -> u64 {
         self.f_fsid as u64
     }
@@ -610,12 +675,18 @@ impl FsMeta for StatFs {
     fn namelen(&self) -> u64 {
         1024
     }
-    #[cfg(target_os = "freebsd")]
+    #[cfg(any(target_os = "freebsd", target_os = "netbsd", target_os = "openbsd"))]
     fn namelen(&self) -> u64 {
         self.f_namemax as u64 // spell-checker:disable-line
     }
     // XXX: should everything just use statvfs?
-    #[cfg(not(any(target_vendor = "apple", target_os = "freebsd", target_os = "linux")))]
+    #[cfg(not(any(
+        target_vendor = "apple",
+        target_os = "freebsd",
+        target_os = "linux",
+        target_os = "netbsd",
+        target_os = "openbsd"
+    )))]
     fn namelen(&self) -> u64 {
         self.f_namemax as u64 // spell-checker:disable-line
     }
@@ -634,9 +705,10 @@ where
                     0 => Ok(buffer),
                     _ => {
                         let errno = IOError::last_os_error().raw_os_error().unwrap_or(0);
-                        Err(CString::from_raw(strerror(errno))
-                            .into_string()
-                            .unwrap_or_else(|_| "Unknown Error".to_owned()))
+                        Err(CStr::from_ptr(strerror(errno))
+                            .to_str()
+                            .map_err(|_| "Error message contains invalid UTF-8".to_owned())?
+                            .to_owned())
                     }
                 }
             }
@@ -829,4 +901,46 @@ mod tests {
         assert_eq!("UNKNOWN (0x1234)", pretty_fstype(0x1234));
         // spell-checker:enable
     }
+
+    #[test]
+    #[cfg(target_os = "linux")]
+    fn test_mountinfo() {
+        // spell-checker:ignore (word) relatime
+        let info = MountInfo::new(
+            LINUX_MOUNTINFO,
+            &"106 109 253:6 / /mnt rw,relatime - xfs /dev/fs0 rw"
+                .split_ascii_whitespace()
+                .collect::<Vec<_>>(),
+        )
+        .unwrap();
+
+        assert_eq!(info.mount_root, "/");
+        assert_eq!(info.mount_dir, "/mnt");
+        assert_eq!(info.mount_option, "rw,relatime");
+        assert_eq!(info.fs_type, "xfs");
+        assert_eq!(info.dev_name, "/dev/fs0");
+
+        // Test parsing with different amounts of optional fields.
+        let info = MountInfo::new(
+            LINUX_MOUNTINFO,
+            &"106 109 253:6 / /mnt rw,relatime master:1 - xfs /dev/fs0 rw"
+                .split_ascii_whitespace()
+                .collect::<Vec<_>>(),
+        )
+        .unwrap();
+
+        assert_eq!(info.fs_type, "xfs");
+        assert_eq!(info.dev_name, "/dev/fs0");
+
+        let info = MountInfo::new(
+            LINUX_MOUNTINFO,
+            &"106 109 253:6 / /mnt rw,relatime master:1 shared:2 - xfs /dev/fs0 rw"
+                .split_ascii_whitespace()
+                .collect::<Vec<_>>(),
+        )
+        .unwrap();
+
+        assert_eq!(info.fs_type, "xfs");
+        assert_eq!(info.dev_name, "/dev/fs0");
+    }
 }
