@@ -9,6 +9,10 @@ static TEST_DIR4: &str = "mkdir_test4/mkdir_test4_1";
 static TEST_DIR5: &str = "mkdir_test5/mkdir_test5_1";
 static TEST_DIR6: &str = "mkdir_test6";
 static TEST_FILE7: &str = "mkdir_test7";
+static TEST_DIR8: &str = "mkdir_test8/mkdir_test8_1/mkdir_test8_2";
+static TEST_DIR9: &str = "mkdir_test9/../mkdir_test9_1/../mkdir_test9_2";
+static TEST_DIR10: &str = "mkdir_test10/.";
+static TEST_DIR11: &str = "mkdir_test11/..";
 
 #[test]
 fn test_mkdir_mkdir() {
@@ -75,7 +79,7 @@ fn test_symbolic_mode() {
 
     ucmd.arg("-m").arg("a=rwx").arg(TEST_DIR1).succeeds();
     let perms = at.metadata(TEST_DIR1).permissions().mode();
-    assert_eq!(perms, 0o40777)
+    assert_eq!(perms, 0o40777);
 }
 
 #[test]
@@ -85,7 +89,7 @@ fn test_symbolic_alteration() {
 
     ucmd.arg("-m").arg("-w").arg(TEST_DIR1).succeeds();
     let perms = at.metadata(TEST_DIR1).permissions().mode();
-    assert_eq!(perms, 0o40555)
+    assert_eq!(perms, 0o40555);
 }
 
 #[test]
@@ -98,5 +102,52 @@ fn test_multi_symbolic() {
         .arg(TEST_DIR1)
         .succeeds();
     let perms = at.metadata(TEST_DIR1).permissions().mode();
-    assert_eq!(perms, 0o40750)
+    assert_eq!(perms, 0o40750);
+}
+
+#[test]
+fn test_recursive_reporting() {
+    new_ucmd!()
+        .arg("-p")
+        .arg("-v")
+        .arg(TEST_DIR8)
+        .succeeds()
+        .stdout_contains("created directory 'mkdir_test8'")
+        .stdout_contains("created directory 'mkdir_test8/mkdir_test8_1'")
+        .stdout_contains("created directory 'mkdir_test8/mkdir_test8_1/mkdir_test8_2'");
+    new_ucmd!().arg("-v").arg(TEST_DIR8).fails().no_stdout();
+    new_ucmd!()
+        .arg("-p")
+        .arg("-v")
+        .arg(TEST_DIR9)
+        .succeeds()
+        .stdout_contains("created directory 'mkdir_test9'")
+        .stdout_contains("created directory 'mkdir_test9/../mkdir_test9_1'")
+        .stdout_contains("created directory 'mkdir_test9/../mkdir_test9_1/../mkdir_test9_2'");
+}
+
+#[test]
+fn test_mkdir_trailing_dot() {
+    let scene2 = TestScenario::new("ls");
+    new_ucmd!()
+        .arg("-p")
+        .arg("-v")
+        .arg("mkdir_test10-2")
+        .succeeds();
+
+    new_ucmd!()
+        .arg("-p")
+        .arg("-v")
+        .arg(TEST_DIR10)
+        .succeeds()
+        .stdout_contains("created directory 'mkdir_test10'");
+
+    new_ucmd!()
+        .arg("-p")
+        .arg("-v")
+        .arg(TEST_DIR11)
+        .succeeds()
+        .stdout_contains("created directory 'mkdir_test11'");
+    let result = scene2.cmd("ls").arg("-al").run();
+    println!("ls dest {}", result.stdout_str());
 }
